"""Defines map of RPC responses by service and method name."""

from ansys.api.edb.v1.bondwire_def_pb2 import Jedec5BondwireDefParametersMessage
from ansys.api.edb.v1.bondwire_pb2 import (
    BondwireTrajMessage,
    BondwireTypeMessage,
    GetCrossSectionTypeMessage,
)
from ansys.api.edb.v1.circle_pb2 import CircleParametersMessage
from ansys.api.edb.v1.component_model_pb2 import ComponentModelTypeMessage
from ansys.api.edb.v1.component_property_pb2 import ComponentPropModelMessage
from ansys.api.edb.v1.connectable_pb2 import LayoutObjTypeMessage
from ansys.api.edb.v1.database_pb2 import GetVersionMessage
from ansys.api.edb.v1.debye_model_pb2 import FrequencyRangeMessage
from ansys.api.edb.v1.die_property_pb2 import DieOrientationMessage, DieTypeMessage
from ansys.api.edb.v1.differential_pair_pb2 import DifferentialPairNetRefsMessage
from ansys.api.edb.v1.edb_error_manager_pb2 import EDBErrorMessages
from ansys.api.edb.v1.edb_messages_pb2 import (
    ComplexMessage,
    ComponentTypeMessage,
    DesignModeMessage,
    DoublesMessage,
    EDBInternalIdMessage,
    EDBObjCollectionMessage,
    EDBObjMessage,
    EDBObjPairMessage,
    HfssExtentInfoMessage,
    IntegersPropertyMessage,
    ProductPropertyIdsMessage,
    StringMapMessage,
    StringPairsMessage,
    StringsMessage,
    TemperatureSettingsMessage,
    TouchstoneExportSettingsMessage,
    ValueMessage,
    ValuePairMessage,
)
from ansys.api.edb.v1.edge_term_pb2 import EdgeParamsMessage, EdgeTypeMessage
from ansys.api.edb.v1.group_pb2 import GroupTypeMessage
from ansys.api.edb.v1.hfss_pi_simulation_settings_pb2 import HFSSPIModelTypeMessage
from ansys.api.edb.v1.hfss_simulation_settings_pb2 import (
    AdaptTypeMessage,
    BasisFunctionOrderMessage,
    BroadbandFrequencyAdaptiveSolutionMessage,
    MultiFrequencyAdaptiveSolutionMessage,
    SingleFrequencyAdaptiveSolutionMessage,
    SolverTypeMessage,
)
from ansys.api.edb.v1.hfss_simulation_setup_pb2 import MeshOperationsMessage
from ansys.api.edb.v1.io_manager_pb2 import ResolvedFuturesMessage
from ansys.api.edb.v1.layer_collection_pb2 import (
    LayerCollectionModeMessage,
    TopBottomStackupLayersMessage,
)
from ansys.api.edb.v1.layer_pb2 import (
    DrawOverrideMessage,
    LayerTypeMessage,
    TopBottomAssociationMessage,
    ZonesMessage,
)
from ansys.api.edb.v1.layout_instance_pb2 import LayoutObjInstancesQueryResultsMessage
from ansys.api.edb.v1.layout_obj_instance_3d_geometry_pb2 import TesselationDataMessage
from ansys.api.edb.v1.layout_obj_instance_pb2 import FetchedLayoutObjInstanceGeometriesMessage
from ansys.api.edb.v1.layout_pb2 import LayoutPortRefTerminalsConnectedMessage
from ansys.api.edb.v1.material_def_pb2 import (
    MaterialDefGetAllPropertiesMessage,
    MaterialDefGetDimensionMessage,
)
from ansys.api.edb.v1.material_property_thermal_modifier_pb2 import (
    MaterialPropertyThermalModifierParamsMessage,
)
from ansys.api.edb.v1.mcad_model_pb2 import McadModelRotationMessage
from ansys.api.edb.v1.multipole_debye_model_pb2 import MultipoleDebyeModelGetParams
from ansys.api.edb.v1.package_def_pb2 import HeatSinkMessage
from ansys.api.edb.v1.padstack_def_data_pb2 import (
    PadstackDefDataConnectionPtMessage,
    PadstackDefDataGetLayerIdsMessage,
    PadstackDefDataGetLayerNamesMessage,
    PadstackDefDataGetSolderBallParamMessage,
    PadstackDefDataPadParametersMessage,
    PadstackDefDataPadstackHoleRangeMessage,
    PadstackDefDataSolderballPlacementMessage,
    PadstackDefDataSolderballShapeMessage,
)
from ansys.api.edb.v1.padstack_inst_term_pb2 import PadstackInstTermParamsMessage
from ansys.api.edb.v1.padstack_instance_pb2 import (
    PadstackInstBackDrillByDepthMessage,
    PadstackInstBackDrillByLayerMessage,
    PadstackInstBackDrillTypeMessage,
    PadstackInstHoleOverridesMessage,
    PadstackInstLayerRangeMessage,
    PadstackInstPositionAndRotationMessage,
)
from ansys.api.edb.v1.path_pb2 import (
    ClipInfoMessage,
    CornerStyleMessage,
    EndCapStyleMessage,
    MiterRatioMessage,
    WidthMessage,
)
from ansys.api.edb.v1.pin_pair_model_pb2 import PinPairRlcMessage
from ansys.api.edb.v1.point_3d_data_pb2 import CPos3DMessage, CPos3DTripleMessage, Point3DMessage
from ansys.api.edb.v1.point_data_pb2 import (
    BoxMessage,
    CircleMessage,
    PointMessage,
    PointPairMessage,
    PointsMessage,
)
from ansys.api.edb.v1.point_term_pb2 import PointTermParamsMessage
from ansys.api.edb.v1.polygon_data_pb2 import (
    IntersectionTypeMessage,
    PolygonDataListMessage,
    PolygonDataMessage,
)
from ansys.api.edb.v1.primitive_pb2 import GetPrimitiveHfssPropMessage, PrimitiveTypeMessage
from ansys.api.edb.v1.q3d_simulation_settings_pb2 import Q3DSolutionOrderMessage
from ansys.api.edb.v1.r_tree_pb2 import GeometrySetsMessage, NearestNeighborResponseMessage
from ansys.api.edb.v1.raptor_x_simulation_settings_pb2 import RaptorXSimSettingsOptionsMessage
from ansys.api.edb.v1.rectangle_pb2 import RectangleParametersMessage
from ansys.api.edb.v1.s_parameter_model_pb2 import SParameterModelMessage
from ansys.api.edb.v1.si_wave_dcir_simulation_settings_pb2 import SourceTermsToGroundMessage
from ansys.api.edb.v1.si_wave_psi_simulation_settings_pb2 import (
    ConductorModelingMessage,
    ErrorToleranceMessage,
    ImprovedLossModelMessage,
    SIWavePSIModelTypeMessage,
    SIWavePSISimulationPreferenceMessage,
    SurfaceRoughnessModelMessage,
)
from ansys.api.edb.v1.si_wave_simulation_settings_pb2 import (
    ACDCMergeModeMessage,
    SParamDCBehaviorMessage,
    SParamExtrapolationMessage,
    SParamInterpolationMessage,
)
from ansys.api.edb.v1.simulation_settings_pb2 import DefeatureModelTypeMessage, ViaModelTypeMessage
from ansys.api.edb.v1.simulation_setup_pb2 import SimulationSetupTypeMessage, SweepDataListMessage
from ansys.api.edb.v1.solder_ball_property_pb2 import DiameterMessage
from ansys.api.edb.v1.spice_model_pb2 import SpiceModelMessage
from ansys.api.edb.v1.stackup_layer_pb2 import (
    EtchNetClassMessage,
    HFSSSolverPropertiesMessage,
    RoughnessModelMessage,
    ViaLayerIdsMessage,
)
from ansys.api.edb.v1.structure_3d_pb2 import ClosureMessage
from ansys.api.edb.v1.term_pb2 import TermParamsMessage, TermProductSolversMessage
from ansys.api.edb.v1.text_pb2 import TextDataMessage
from ansys.api.edb.v1.variable_server_pb2 import GetVariableNamesMessage
from ansys.api.edb.v1.via_layer_pb2 import OxideLayersMessage
from ansys.api.edb.v1.voltage_regulator_pb2 import PowerModuleMessage, PowerModulesMessage
from google.protobuf.empty_pb2 import Empty
from google.protobuf.wrappers_pb2 import (
    BoolValue,
    DoubleValue,
    FloatValue,
    Int32Value,
    Int64Value,
    StringValue,
    UInt64Value,
)

rpc_response_map = {
    "ansys.api.edb.v1.ArcDataService": {
        "GetHeight": FloatValue,
        "GetCenter": PointMessage,
        "GetMidpoint": PointMessage,
        "GetRadius": FloatValue,
        "GetBoundingBox": BoxMessage,
        "GetAngle": FloatValue,
        "ClosestPoints": BoxMessage,
    },
    "ansys.api.edb.v1.BoardBendDefService": {
        "Create": EDBObjMessage,
        "GetBoundaryPrim": EDBObjMessage,
        "GetBendMiddle": PointPairMessage,
        "SetBendMiddle": Empty,
        "GetRadius": ValueMessage,
        "SetRadius": Empty,
        "GetAngle": ValueMessage,
        "SetAngle": Empty,
        "GetBentRegions": PolygonDataListMessage,
    },
    "ansys.api.edb.v1.BondwireService": {
        "Create": EDBObjMessage,
        "GetMaterial": StringValue,
        "SetMaterial": Empty,
        "GetType": BondwireTypeMessage,
        "SetType": Empty,
        "GetCrossSectionType": GetCrossSectionTypeMessage,
        "SetCrossSectionType": Empty,
        "GetCrossSectionHeight": ValueMessage,
        "SetCrossSectionHeight": Empty,
        "GetDefinitionName": StringValue,
        "SetDefinitionName": Empty,
        "GetStartElevation": EDBObjPairMessage,
        "SetStartElevation": Empty,
        "GetEndElevation": EDBObjPairMessage,
        "SetEndElevation": Empty,
        "GetTraj": BondwireTrajMessage,
        "SetTraj": Empty,
        "GetWidthValue": ValueMessage,
        "SetWidthValue": Empty,
    },
    "ansys.api.edb.v1.BondwireDefService": {"Delete": Empty, "GetName": StringValue},
    "ansys.api.edb.v1.ApdBondwireDefService": {
        "Create": EDBObjMessage,
        "LoadDefinitionsFromFile": Empty,
        "FindByName": EDBObjMessage,
        "GetParameters": StringValue,
        "SetParameters": Empty,
    },
    "ansys.api.edb.v1.Jedec4BondwireDefService": {
        "Create": EDBObjMessage,
        "FindByName": EDBObjMessage,
        "GetParameters": ValueMessage,
        "SetParameters": Empty,
    },
    "ansys.api.edb.v1.Jedec5BondwireDefService": {
        "Create": EDBObjMessage,
        "FindByName": EDBObjMessage,
        "GetParameters": Jedec5BondwireDefParametersMessage,
        "SetParameters": Empty,
    },
    "ansys.api.edb.v1.BundleTerminalService": {
        "Create": EDBObjMessage,
        "Ungroup": Empty,
        "GetTerminals": EDBObjCollectionMessage,
        "StreamTerminals": EDBObjCollectionMessage,
    },
    "ansys.api.edb.v1.CellService": {
        "Create": EDBObjMessage,
        "GetLayout": EDBObjMessage,
        "GetFlattenedLayout": EDBObjMessage,
        "Find": EDBObjMessage,
        "Delete": Empty,
        "GetDatabase": EDBObjMessage,
        "IsFootprint": BoolValue,
        "IsBlackBox": BoolValue,
        "SetBlackBox": Empty,
        "GetSuppressPads": BoolValue,
        "SetSuppressPads": Empty,
        "GetAntiPadsAlwaysOn": BoolValue,
        "SetAntiPadsAlwaysOn": Empty,
        "GetAntiPadsOption": Int64Value,
        "SetAntiPadsOption": Empty,
        "IsSymbolicFootprint": BoolValue,
        "GetName": StringValue,
        "SetName": Empty,
        "GetDesignMode": DesignModeMessage,
        "SetDesignMode": Empty,
        "GetHfssExtentInfo": HfssExtentInfoMessage,
        "SetHfssExtentInfo": Empty,
        "GetTemperatureSettings": TemperatureSettingsMessage,
        "SetTemperatureSettings": Empty,
        "GetTouchstoneExportSettings": TouchstoneExportSettingsMessage,
        "SetTouchstoneExportSettings": Empty,
        "CutOut": EDBObjMessage,
        "GetProductPropertyIds": ProductPropertyIdsMessage,
        "GetProductProperty": StringValue,
        "SetProductProperty": Empty,
        "DeleteSimulationSetup": Empty,
        "GetSimulationSetups": EDBObjCollectionMessage,
        "StreamSimulationSetups": EDBObjCollectionMessage,
        "GenerateAutoHFSSRegions": Empty,
        "GenerateViaSmartBox": PolygonDataListMessage,
        "ApplyTechnology": Empty,
    },
    "ansys.api.edb.v1.CellInstanceService": {
        "Create": EDBObjMessage,
        "CreateWithComponent": EDBObjMessage,
        "FindByName": EDBObjMessage,
        "GetReferenceLayout": EDBObjMessage,
        "GetTermInsts": EDBObjCollectionMessage,
        "StreamTermInsts": EDBObjCollectionMessage,
        "GetIs3DPlacement": BoolValue,
        "Set3DPlacement": Empty,
        "Get3DTransform": EDBObjMessage,
        "Set3DTransform": Empty,
        "GetParameterOverride": ValueMessage,
        "SetParameterOverride": Empty,
    },
    "ansys.api.edb.v1.CircleService": {
        "Create": EDBObjMessage,
        "Render": PolygonDataMessage,
        "GetParameters": CircleParametersMessage,
        "SetParameters": Empty,
    },
    "ansys.api.edb.v1.ComponentDefService": {
        "Create": EDBObjMessage,
        "FindByName": EDBObjMessage,
        "SetName": Empty,
        "GetName": StringValue,
        "SetFootprintCell": Empty,
        "GetFootprintCell": EDBObjMessage,
        "GetComponentModels": EDBObjCollectionMessage,
        "StreamComponentModels": EDBObjCollectionMessage,
        "GetComponentPins": EDBObjCollectionMessage,
        "StreamComponentPins": EDBObjCollectionMessage,
        "AddComponentModel": Empty,
        "RemoveComponentModel": Empty,
        "ReorderPins": Empty,
        "RemovePin": Empty,
        "GetProductProperty": StringValue,
        "SetProductProperty": Empty,
        "GetProductPropertyIds": ProductPropertyIdsMessage,
    },
    "ansys.api.edb.v1.ComponentGroupService": {
        "Create": EDBObjMessage,
        "GetNumberOfPins": UInt64Value,
        "GetComponentProperty": EDBObjMessage,
        "SetComponentProperty": Empty,
        "GetComponentType": ComponentTypeMessage,
        "SetComponentType": Empty,
        "FindByDef": EDBObjCollectionMessage,
    },
    "ansys.api.edb.v1.ComponentModelService": {
        "SetReferenceFile": Empty,
        "GetReferenceFile": StringValue,
        "FindByName": EDBObjMessage,
        "FindById": EDBObjMessage,
        "GetName": StringValue,
        "GetType": ComponentModelTypeMessage,
        "GetId": UInt64Value,
    },
    "ansys.api.edb.v1.NPortComponentModelService": {"Create": EDBObjMessage},
    "ansys.api.edb.v1.DynamicLinkComponentModelService": {
        "Create": EDBObjMessage,
        "SetDesignName": Empty,
        "GetDesignName": StringValue,
    },
    "ansys.api.edb.v1.ComponentPinService": {
        "Create": EDBObjMessage,
        "FindByName": EDBObjMessage,
        "SetName": Empty,
        "GetName": StringValue,
        "GetNumber": UInt64Value,
        "GetComponentDef": EDBObjMessage,
    },
    "ansys.api.edb.v1.ComponentPropertyService": {
        "Clone": EDBObjMessage,
        "GetPackageMountingOffset": ValueMessage,
        "SetPackageMountingOffset": Empty,
        "GetPackageDef": EDBObjMessage,
        "SetPackageDef": Empty,
        "GetModel": ComponentPropModelMessage,
        "SetModel": Empty,
    },
    "ansys.api.edb.v1.ConnectableService": {
        "GetObjType": LayoutObjTypeMessage,
        "FindByIdAndType": EDBObjMessage,
        "GetId": EDBInternalIdMessage,
        "GetComponent": EDBObjMessage,
        "GetGroup": EDBObjMessage,
        "SetGroup": Empty,
        "GetNet": EDBObjMessage,
        "SetNet": Empty,
    },
    "ansys.api.edb.v1.DatabaseService": {
        "Create": EDBObjMessage,
        "Open": EDBObjMessage,
        "Delete": Empty,
        "Save": Empty,
        "Close": Empty,
        "IsReadOnly": BoolValue,
        "GetTopCircuits": EDBObjCollectionMessage,
        "StreamTopCircuits": EDBObjCollectionMessage,
        "GetId": Int64Value,
        "FindById": EDBObjMessage,
        "SaveAs": Empty,
        "GetVersionByRelease": StringValue,
        "GetDirectory": StringValue,
        "GetProductProperty": StringValue,
        "SetProductProperty": Empty,
        "GetProductPropertyIds": ProductPropertyIdsMessage,
        "ImportMaterialFromControlFile": Empty,
        "GetVersion": GetVersionMessage,
        "Scale": Empty,
        "GetSource": StringValue,
        "SetSource": Empty,
        "GetSourceVersion": StringValue,
        "SetSourceVersion": Empty,
        "CopyCells": EDBObjCollectionMessage,
        "GetDefinitionObjs": EDBObjCollectionMessage,
        "StreamDefinitionObjs": EDBObjCollectionMessage,
        "TopCircuitCells": EDBObjCollectionMessage,
        "GetCircuits": EDBObjCollectionMessage,
        "StreamCircuits": EDBObjCollectionMessage,
        "GetFootprints": EDBObjCollectionMessage,
        "StreamFootprints": EDBObjCollectionMessage,
        "Encrypt": Empty,
    },
    "ansys.api.edb.v1.DatasetDefService": {
        "Create": EDBObjMessage,
        "FindByName": EDBObjMessage,
        "GetName": StringValue,
        "SetName": Empty,
        "GetData": PointsMessage,
        "SetData": Empty,
    },
    "ansys.api.edb.v1.DebyeModelService": {
        "Create": EDBObjMessage,
        "GetFrequencyRange": FrequencyRangeMessage,
        "SetFrequencyRange": Empty,
        "GetRelativePermitivityAtHighLowFrequency": FrequencyRangeMessage,
        "SetRelativePermitivityAtHighLowFrequency": Empty,
        "IsRelativePermitivityEnabledAtOpticalFrequency": BoolValue,
        "SetRelativePermitivityEnabledAtOpticalFrequency": Empty,
        "GetRelativePermitivityAtOpticalFrequency": DoubleValue,
        "SetRelativePermitivityAtOpticalFrequency": Empty,
        "UseDCConductivity": BoolValue,
        "SetUseDCConductivity": Empty,
        "GetDCConductivity": DoubleValue,
        "SetDCConductivity": Empty,
        "GetLossTangentAtHighLowFrequency": FrequencyRangeMessage,
        "SetLossTangentAtHighLowFrequency": Empty,
    },
    "ansys.api.edb.v1.DielectricMaterialModelService": {"GetType": Int32Value},
    "ansys.api.edb.v1.DiePropertyService": {
        "Create": EDBObjMessage,
        "Clone": EDBObjMessage,
        "GetDieType": DieTypeMessage,
        "SetDieType": Empty,
        "GetHeight": ValueMessage,
        "SetHeight": Empty,
        "GetOrientation": DieOrientationMessage,
        "SetOrientation": Empty,
    },
    "ansys.api.edb.v1.DifferentialPairService": {
        "Create": EDBObjMessage,
        "FindByName": EDBObjMessage,
        "GetDifferentialPair": DifferentialPairNetRefsMessage,
        "SetDifferentialPair": Empty,
    },
    "ansys.api.edb.v1.DjordjecvicSarkarModelService": {
        "Create": EDBObjMessage,
        "GetFrequency": DoubleValue,
        "SetFrequency": Empty,
        "GetRelativePermitivityAtFrequency": DoubleValue,
        "SetRelativePermitivityAtFrequency": Empty,
        "GetLossTangentAtFrequency": DoubleValue,
        "SetLossTangentAtFrequency": Empty,
        "GetHighFrequencyCorner": DoubleValue,
        "SetHighFrequencyCorner": Empty,
        "UseDCRelativePermitivity": BoolValue,
        "SetUseDCRelativePermitivity": Empty,
        "GetDCRelativePermitivity": DoubleValue,
        "SetDCRelativePermitivity": Empty,
        "GetDCConductivity": DoubleValue,
        "SetDCConductivity": Empty,
    },
    "ansys.api.edb.v1.EDBErrorManagerService": {"GetErrors": EDBErrorMessages},
    "ansys.api.edb.v1.EdgeService": {
        "GetEdgeType": EdgeTypeMessage,
        "Create": EDBObjMessage,
        "GetParameters": EdgeParamsMessage,
    },
    "ansys.api.edb.v1.EdgeTerminalService": {
        "Create": EDBObjMessage,
        "GetEdges": EDBObjCollectionMessage,
        "StreamEdges": EDBObjCollectionMessage,
        "SetEdges": Empty,
    },
    "ansys.api.edb.v1.ExtendedNetService": {
        "Create": EDBObjMessage,
        "FindByName": EDBObjMessage,
        "RemoveNet": Empty,
        "AddNet": Empty,
        "RemoveAllNets": Empty,
    },
    "ansys.api.edb.v1.GroupService": {
        "Create": EDBObjMessage,
        "FindByName": EDBObjMessage,
        "AddMember": Empty,
        "RemoveMember": Empty,
        "Ungroup": Empty,
        "GetMembers": EDBObjCollectionMessage,
        "StreamMembers": EDBObjCollectionMessage,
        "GetGroupType": GroupTypeMessage,
    },
    "ansys.api.edb.v1.HFSSPIGeneralSettingsService": {
        "GetHFSSPIModelType": HFSSPIModelTypeMessage,
        "SetHFSSPIModelType": Empty,
        "GetUseAutoMeshRegion": BoolValue,
        "SetUseAutoMeshRegion": Empty,
        "GetUseMeshRegion": BoolValue,
        "SetUseMeshRegion": Empty,
        "GetMeshRegionName": StringValue,
        "SetMeshRegionName": Empty,
    },
    "ansys.api.edb.v1.HFSSPIAdvancedSettingsService": {
        "GetICModeAutoResolution": BoolValue,
        "SetICModeAutoResolution": Empty,
        "GetICModeLength": StringValue,
        "SetICModeLength": Empty,
        "GetSmallPlaneArea": StringValue,
        "SetSmallPlaneArea": Empty,
        "GetZeroMetalLayerThickness": StringValue,
        "SetZeroMetalLayerThickness": Empty,
    },
    "ansys.api.edb.v1.HFSSPISolverSettingsService": {
        "GetEnhancedLowFrequencyAccuracy": BoolValue,
        "SetEnhancedLowFrequencyAccuracy": Empty,
        "GetViaAreaCutoffCircElems": StringValue,
        "SetViaAreaCutoffCircElems": Empty,
    },
    "ansys.api.edb.v1.HFSSGeneralSettingsService": {
        "GetSingleFrequencyAdaptiveSolution": SingleFrequencyAdaptiveSolutionMessage,
        "SetSingleFrequencyAdaptiveSolution": Empty,
        "GetMultiFrequencyAdaptiveSolution": MultiFrequencyAdaptiveSolutionMessage,
        "SetMultiFrequencyAdaptiveSolution": Empty,
        "GetBroadbandFrequencyAdaptiveSolution": BroadbandFrequencyAdaptiveSolutionMessage,
        "SetBroadbandFrequencyAdaptiveSolution": Empty,
        "GetSaveFieldsFlag": BoolValue,
        "SetSaveFieldsFlag": Empty,
        "GetUseMeshRegion": BoolValue,
        "SetUseMeshRegion": Empty,
        "GetMeshRegionName": StringValue,
        "SetMeshRegionName": Empty,
        "GetUseParallelRefinement": BoolValue,
        "SetUseParallelRefinement": Empty,
        "GetAdaptType": AdaptTypeMessage,
        "SetAdaptType": Empty,
        "GetSaveRadFieldsOnlyFlag": BoolValue,
        "SetSaveRadFieldsOnlyFlag": Empty,
    },
    "ansys.api.edb.v1.HFSSOptionsSettingsService": {
        "GetMaxRefinementPerPass": UInt64Value,
        "SetMaxRefinementPerPass": Empty,
        "GetMinPasses": UInt64Value,
        "SetMinPasses": Empty,
        "GetMinConvergedPasses": UInt64Value,
        "SetMinConvergedPasses": Empty,
        "GetUseMaxRefinement": BoolValue,
        "SetUseMaxRefinement": Empty,
        "GetBasisFunctionOrder": BasisFunctionOrderMessage,
        "SetBasisFunctionOrder": Empty,
        "GetSolveInsideMetalBasis": BasisFunctionOrderMessage,
        "SetSolveInsideMetalBasis": Empty,
        "GetSolverTypeOrder": SolverTypeMessage,
        "SetSolverTypeOrder": Empty,
        "GetRelativeResidual": DoubleValue,
        "SetRelativeResidual": Empty,
        "GetUseShellElements": BoolValue,
        "SetUseShellElements": Empty,
        "GetEnhancedLowFrequencyAccuracy": BoolValue,
        "SetEnhancedLowFrequencyAccuracy": Empty,
    },
    "ansys.api.edb.v1.HFSSAdvancedSettingsService": {
        "GetICModeAutoResolution": BoolValue,
        "SetICModeAutoResolution": Empty,
        "GetICModeLength": StringValue,
        "SetICModeLength": Empty,
    },
    "ansys.api.edb.v1.HFSSAdvancedMeshingSettingsService": {
        "GetLayerAlignment": StringValue,
        "SetLayerAlignment": Empty,
    },
    "ansys.api.edb.v1.HFSSSolverSettingsService": {
        "GetMaxDeltaZ0": DoubleValue,
        "SetMaxDeltaZ0": Empty,
        "GetSetTrianglesForWaveport": BoolValue,
        "SetSetTrianglesForWaveport": Empty,
        "GetMinTrianglesForWavePort": UInt64Value,
        "SetMinTrianglesForWavePort": Empty,
        "GetMaxTrianglesForWavePort": UInt64Value,
        "SetMaxTrianglesForWavePort": Empty,
        "GetIntraPlaneCouplingEnabled": BoolValue,
        "SetIntraPlaneCouplingEnabled": Empty,
    },
    "ansys.api.edb.v1.DCRSettingsService": {
        "GetMaxPasses": UInt64Value,
        "SetMaxPasses": Empty,
        "GetMinPasses": UInt64Value,
        "SetMinPasses": Empty,
        "GetMinConvergedPasses": UInt64Value,
        "SetMinConvergedPasses": Empty,
        "GetPercentError": DoubleValue,
        "SetPercentError": Empty,
        "GetPercentRefinementPerPass": DoubleValue,
        "SetPercentRefinementPerPass": Empty,
    },
    "ansys.api.edb.v1.HfssSimulationSetupService": {
        "GetMeshOperations": MeshOperationsMessage,
        "SetMeshOperations": Empty,
    },
    "ansys.api.edb.v1.HierarchyObjectService": {
        "GetTransform": EDBObjMessage,
        "SetTransform": Empty,
        "GetName": StringValue,
        "SetName": Empty,
        "GetComponent": EDBObjMessage,
        "GetPlacementLayer": EDBObjMessage,
        "SetPlacementLayer": Empty,
        "GetLocation": PointMessage,
        "SetLocation": Empty,
        "GetSolveIndependentPreference": BoolValue,
        "SetSolveIndependentPreference": Empty,
    },
    "ansys.api.edb.v1.ICComponentPropertyService": {
        "Create": EDBObjMessage,
        "SetSolderBallProperty": Empty,
        "GetSolderBallProperty": EDBObjMessage,
        "SetDieProperty": Empty,
        "GetDieProperty": EDBObjMessage,
        "SetPortProperty": Empty,
        "GetPortProperty": EDBObjMessage,
    },
    "ansys.api.edb.v1.InstArrayService": {
        "Create": EDBObjMessage,
        "FindByName": EDBObjMessage,
        "GetOrig": PointMessage,
        "GetXAxis": PointMessage,
        "SetXAxis": Empty,
        "GetYAxis": PointMessage,
        "SetYAxis": Empty,
        "GetXCount": ValueMessage,
        "SetXCount": Empty,
        "GetYCount": ValueMessage,
        "SetYCount": Empty,
        "Decompose": Empty,
    },
    "ansys.api.edb.v1.IOComponentPropertyService": {
        "Create": EDBObjMessage,
        "SetSolderBallProperty": Empty,
        "GetSolderBallProperty": EDBObjMessage,
        "SetPortProperty": Empty,
        "GetPortProperty": EDBObjMessage,
    },
    "ansys.api.edb.v1.IOManagerService": {
        "FlushBuffer": ResolvedFuturesMessage,
        "FlushBufferStream": ResolvedFuturesMessage,
        "FlushBufferNoFutureResolutionStream": Empty,
        "RefreshCache": EDBObjCollectionMessage,
        "EnableCache": Empty,
    },
    "ansys.api.edb.v1.LayerService": {
        "Create": EDBObjMessage,
        "GetLayerType": LayerTypeMessage,
        "SetLayerType": Empty,
        "IsViaLayer": BoolValue,
        "GetName": StringValue,
        "SetName": Empty,
        "Clone": EDBObjMessage,
        "GetLayerId": Int64Value,
        "GetTopBottomAssociation": TopBottomAssociationMessage,
        "SetTopBottomAssociation": Empty,
        "GetColor": UInt64Value,
        "SetColor": Empty,
        "GetVisibilityMask": UInt64Value,
        "SetVisibilityMask": Empty,
        "GetLocked": BoolValue,
        "SetLocked": Empty,
        "GetTransparency": UInt64Value,
        "SetTransparency": Empty,
        "GetDrawOverride": DrawOverrideMessage,
        "SetDrawOverride": Empty,
        "GetProductProperty": StringValue,
        "SetProductProperty": Empty,
        "GetProductPropertyIds": ProductPropertyIdsMessage,
        "IsInZone": BoolValue,
        "SetIsInZone": Empty,
        "GetZones": ZonesMessage,
        "GetZone": Int64Value,
    },
    "ansys.api.edb.v1.LayerCollectionService": {
        "Create": EDBObjMessage,
        "Clone": EDBObjMessage,
        "GetMode": LayerCollectionModeMessage,
        "SetMode": Empty,
        "AddLayers": Empty,
        "ImportFromControlFile": Empty,
        "AddLayer": EDBObjMessage,
        "IsValid": BoolValue,
        "FindByName": EDBObjMessage,
        "GetTopBottomStackupLayers": TopBottomStackupLayersMessage,
        "GetLayers": EDBObjCollectionMessage,
        "StreamLayers": EDBObjCollectionMessage,
        "GetProductProperty": StringValue,
        "SetProductProperty": Empty,
        "GetProductPropertyIds": ProductPropertyIdsMessage,
        "MergeDielectrics": EDBObjMessage,
        "GetZoneIds": ZonesMessage,
        "GetZoneName": StringValue,
        "SetZoneName": Empty,
        "InsertZone": Int64Value,
        "RemoveZone": Empty,
        "SimplifyDielectricsForPhi": EDBObjCollectionMessage,
        "AddZoneToLayer": Empty,
    },
    "ansys.api.edb.v1.LayerMapService": {
        "Create": EDBObjMessage,
        "Clear": Empty,
        "SetMapping": Empty,
        "GetMappingForward": ProductPropertyIdsMessage,
        "GetMappingBackward": ProductPropertyIdsMessage,
    },
    "ansys.api.edb.v1.LayoutService": {
        "GetCell": EDBObjMessage,
        "GetLayerCollection": EDBObjMessage,
        "SetLayerCollection": Empty,
        "GetItems": EDBObjCollectionMessage,
        "StreamItems": EDBObjCollectionMessage,
        "GetExpandedExtentFromNets": PolygonDataMessage,
        "ConvertPrimitivesToVias": Empty,
        "ArePortReferenceTerminalsConnected": LayoutPortRefTerminalsConnectedMessage,
        "GetZonePrimitives": EDBObjCollectionMessage,
        "StreamZonePrimitives": EDBObjCollectionMessage,
        "SetFixedZonePrimitives": Empty,
        "GetFixedZonePrimitive": EDBObjMessage,
        "GetBoardBendDefs": EDBObjCollectionMessage,
        "StreamBoardBendDefs": EDBObjCollectionMessage,
        "SynchronizeBendManager": Empty,
        "GetLayoutInstance": EDBObjMessage,
        "ReconstructArcs": Empty,
        "UnitePrimitives": Empty,
        "GroupVias": Empty,
        "SnapVias": Empty,
        "SnapPrimitives": Empty,
        "CreateMeshRegion": Empty,
        "CompressPrimitives": Empty,
        "FlattenCellInstances": Empty,
        "RemoveHolesExceptOnCriticalNets": Empty,
        "RemoveUnusedConductingLayers": Empty,
    },
    "ansys.api.edb.v1.LayoutComponentService": {
        "ExportLayoutComponent": Empty,
        "ImportLayoutComponent": EDBObjMessage,
        "GetCellInstance": EDBObjMessage,
    },
    "ansys.api.edb.v1.LayoutInstanceService": {
        "Refresh": Empty,
        "QueryLayoutObjInstances": LayoutObjInstancesQueryResultsMessage,
        "StreamLayoutObjInstancesQuery": LayoutObjInstancesQueryResultsMessage,
        "GetLayoutObjInstanceInContext": EDBObjMessage,
        "GetConnectedObjects": EDBObjCollectionMessage,
        "StreamConnectedObjects": EDBObjCollectionMessage,
    },
    "ansys.api.edb.v1.LayoutInstanceContextService": {
        "GetLayout": EDBObjMessage,
        "GetBBox": BoxMessage,
        "IsTopOrBlackBox": BoolValue,
        "GetTopOrBlackBox": EDBObjMessage,
        "GetPlacementElevation": DoubleValue,
        "Is3DPlacement": BoolValue,
        "GetTransformation": EDBObjMessage,
        "GetTransformationBetweenContexts": EDBObjMessage,
    },
    "ansys.api.edb.v1.LayoutObjService": {
        "GetLayout": EDBObjMessage,
        "Delete": Empty,
        "GetProductProperty": StringValue,
        "SetProductProperty": Empty,
        "GetProductPropertyIds": ProductPropertyIdsMessage,
    },
    "ansys.api.edb.v1.LayoutObjInstanceService": {
        "GetLayers": EDBObjCollectionMessage,
        "StreamLayers": EDBObjCollectionMessage,
        "GetGeometries": FetchedLayoutObjInstanceGeometriesMessage,
        "GetContext": StringsMessage,
        "GetLayoutInstanceContext": EDBObjMessage,
        "GetLayoutObj": EDBObjMessage,
        "GetBBox": BoxMessage,
    },
    "ansys.api.edb.v1.LayoutObjInstance2DGeometryService": {
        "IsNegative": BoolValue,
        "GetPolygonData": PolygonDataMessage,
    },
    "ansys.api.edb.v1.LayoutObjInstance3DGeometryService": {
        "GetTesselationData": TesselationDataMessage
    },
    "ansys.api.edb.v1.LayoutObjInstanceGeometryService": {
        "GetMaterial": StringValue,
        "GetColor": Int64Value,
    },
    "ansys.api.edb.v1.MaterialDefService": {
        "Create": EDBObjMessage,
        "SetProperty": Empty,
        "FindByName": EDBObjMessage,
        "Delete": Empty,
        "GetProperty": ValueMessage,
        "GetAllProperties": MaterialDefGetAllPropertiesMessage,
        "RemoveProperty": Empty,
        "GetName": StringValue,
        "GetDielectricMaterialModel": EDBObjMessage,
        "SetDielectricMaterialModel": Empty,
        "GetDimensions": MaterialDefGetDimensionMessage,
        "SetThermalModifier": Empty,
        "SetAnisotropicThermalModifier": Empty,
        "GetThermalModifier": EDBObjMessage,
        "GetAnisotropicThermalModifier": EDBObjMessage,
    },
    "ansys.api.edb.v1.MaterialPropertyThermalModifierService": {
        "Create": EDBObjMessage,
        "GetQuadraticModelParams": MaterialPropertyThermalModifierParamsMessage,
        "GetThermalModifierExpression": ValueMessage,
    },
    "ansys.api.edb.v1.McadModelService": {
        "CreateStride": EDBObjMessage,
        "CreateHfss": EDBObjMessage,
        "Create3dComp": EDBObjMessage,
        "IsMcad": BoolValue,
        "IsMcadStride": BoolValue,
        "IsMcadHfss": BoolValue,
        "IsMcad3dComp": BoolValue,
        "GetOrigin": Point3DMessage,
        "SetOrigin": Empty,
        "GetRotation": McadModelRotationMessage,
        "SetRotation": Empty,
        "GetScale": DoubleValue,
        "SetScale": Empty,
        "GetMaterial": StringValue,
        "SetMaterial": Empty,
        "GetVisible": BoolValue,
        "SetVisible": Empty,
        "GetModeled": BoolValue,
        "SetModeled": Empty,
        "GetCellInst": EDBObjMessage,
        "GetPartCount": Int64Value,
        "GetPartName": StringValue,
        "GetPartIndex": Int64Value,
        "GetModelName": StringValue,
        "GetDesignName": StringValue,
    },
    "ansys.api.edb.v1.ModelService": {"Clone": EDBObjMessage},
    "ansys.api.edb.v1.MultipoleDebyeModelService": {
        "Create": EDBObjMessage,
        "GetParameters": MultipoleDebyeModelGetParams,
        "SetParameters": Empty,
    },
    "ansys.api.edb.v1.NetService": {
        "Create": EDBObjMessage,
        "FindByName": EDBObjMessage,
        "GetName": StringValue,
        "SetName": Empty,
        "GetIsPowerGround": BoolValue,
        "SetIsPowerGround": Empty,
        "GetLayoutObjects": EDBObjCollectionMessage,
        "StreamLayoutObjects": EDBObjCollectionMessage,
    },
    "ansys.api.edb.v1.NetlistModelService": {
        "Create": EDBObjMessage,
        "GetNetlist": StringValue,
        "SetNetlist": Empty,
    },
    "ansys.api.edb.v1.NetClassService": {
        "Create": EDBObjMessage,
        "FindByName": EDBObjMessage,
        "GetName": StringValue,
        "SetName": Empty,
        "GetDescription": StringValue,
        "SetDescription": Empty,
        "IsPowerGround": BoolValue,
        "GetNets": EDBObjCollectionMessage,
        "StreamNets": EDBObjCollectionMessage,
        "AddNet": Empty,
        "RemoveNet": Empty,
        "ContainsNet": BoolValue,
    },
    "ansys.api.edb.v1.PackageDefService": {
        "Create": EDBObjMessage,
        "Delete": Empty,
        "FindByName": EDBObjMessage,
        "FindByEDBUId": EDBObjMessage,
        "GetName": StringValue,
        "SetName": Empty,
        "GetExteriorBoundary": PolygonDataMessage,
        "SetExteriorBoundary": Empty,
        "GetHeight": ValueMessage,
        "SetHeight": Empty,
        "GetOperatingPower": ValueMessage,
        "SetOperatingPower": Empty,
        "GetMaximumPower": ValueMessage,
        "SetMaximumPower": Empty,
        "GetTherm_Cond": ValueMessage,
        "SetTherm_Cond": Empty,
        "GetTheta_JB": ValueMessage,
        "SetTheta_JB": Empty,
        "GetTheta_JC": ValueMessage,
        "SetTheta_JC": Empty,
        "GetHeatSink": HeatSinkMessage,
        "SetHeatSink": Empty,
        "GetProductProperty": StringValue,
        "SetProductProperty": Empty,
        "GetProductPropertyIds": ProductPropertyIdsMessage,
    },
    "ansys.api.edb.v1.PadstackDefService": {
        "Create": EDBObjMessage,
        "Delete": Empty,
        "FindByName": EDBObjMessage,
        "GetName": StringValue,
        "GetData": EDBObjMessage,
        "SetData": Empty,
    },
    "ansys.api.edb.v1.PadstackDefDataService": {
        "Create": EDBObjMessage,
        "GetMaterial": StringValue,
        "SetMaterial": Empty,
        "GetLayerNames": PadstackDefDataGetLayerNamesMessage,
        "GetLayerIds": PadstackDefDataGetLayerIdsMessage,
        "AddLayers": Empty,
        "GetPadParameters": PadstackDefDataPadParametersMessage,
        "SetPadParameters": Empty,
        "GetHoleRange": PadstackDefDataPadstackHoleRangeMessage,
        "SetHoleRange": Empty,
        "GetPlatingPercentage": ValueMessage,
        "SetPlatingPercentage": Empty,
        "GetSolderBallShape": PadstackDefDataSolderballShapeMessage,
        "SetSolderBallShape": Empty,
        "GetSolderBallPlacement": PadstackDefDataSolderballPlacementMessage,
        "SetSolderBallPlacement": Empty,
        "GetSolderBallParam": PadstackDefDataGetSolderBallParamMessage,
        "SetSolderBallParam": Empty,
        "GetSolderBallMaterial": StringValue,
        "SetSolderBallMaterial": Empty,
        "GetConnectionPt": PadstackDefDataConnectionPtMessage,
        "SetConnectionPt": Empty,
    },
    "ansys.api.edb.v1.PadstackInstanceService": {
        "Create": EDBObjMessage,
        "GetPadstackDef": EDBObjMessage,
        "GetName": StringValue,
        "SetName": Empty,
        "GetPositionAndRotation": PadstackInstPositionAndRotationMessage,
        "SetPositionAndRotation": Empty,
        "GetLayerRange": PadstackInstLayerRangeMessage,
        "SetLayerRange": Empty,
        "GetSolderBallLayer": EDBObjMessage,
        "SetSolderBallLayer": Empty,
        "GetLayerMap": EDBObjMessage,
        "SetLayerMap": Empty,
        "GetHoleOverrides": PadstackInstHoleOverridesMessage,
        "SetHoleOverrides": Empty,
        "GetIsLayoutPin": BoolValue,
        "SetIsLayoutPin": Empty,
        "GetBackDrillType": PadstackInstBackDrillTypeMessage,
        "GetBackDrillByLayer": PadstackInstBackDrillByLayerMessage,
        "GetBackDrillByDepth": PadstackInstBackDrillByDepthMessage,
        "SetBackDrillByLayer": Empty,
        "SetBackDrillByDepth": Empty,
        "GetPadstackInstanceTerminal": EDBObjMessage,
        "IsInPinGroup": BoolValue,
        "GetPinGroups": EDBObjCollectionMessage,
        "StreamPinGroups": EDBObjCollectionMessage,
    },
    "ansys.api.edb.v1.PadstackInstanceTerminalService": {
        "Create": EDBObjMessage,
        "GetParameters": PadstackInstTermParamsMessage,
        "SetParameters": Empty,
    },
    "ansys.api.edb.v1.PathService": {
        "Create": EDBObjMessage,
        "Render": PolygonDataMessage,
        "GetPolygonData": PolygonDataMessage,
        "GetCenterLine": PolygonDataMessage,
        "SetCenterLine": Empty,
        "GetEndCapStyle": EndCapStyleMessage,
        "SetEndCapStyle": Empty,
        "GetClipInfo": ClipInfoMessage,
        "SetClipInfo": Empty,
        "GetCornerStyle": CornerStyleMessage,
        "SetCornerStyle": Empty,
        "GetWidth": WidthMessage,
        "SetWidth": Empty,
        "GetMiterRatio": MiterRatioMessage,
        "SetMiterRatio": Empty,
    },
    "ansys.api.edb.v1.PinGroupService": {
        "Create": EDBObjMessage,
        "FindByName": EDBObjMessage,
        "GetUniqueName": StringValue,
        "GetName": StringValue,
        "AddPins": Empty,
        "RemovePins": Empty,
        "GetPins": EDBObjCollectionMessage,
        "StreamPins": EDBObjCollectionMessage,
        "GetPinGroupTerminal": EDBObjMessage,
    },
    "ansys.api.edb.v1.PinGroupTerminalService": {
        "Create": EDBObjMessage,
        "GetPinGroup": EDBObjMessage,
        "SetPinGroup": Empty,
        "GetLayer": EDBObjMessage,
        "SetLayer": Empty,
    },
    "ansys.api.edb.v1.PinPairModelService": {
        "Create": EDBObjMessage,
        "GetRlc": PinPairRlcMessage,
        "SetRlc": Empty,
        "DeleteRlc": Empty,
        "GetPinPairs": StringPairsMessage,
    },
    "ansys.api.edb.v1.PointDataService": {
        "Rotate": PointMessage,
        "ClosestPoint": PointMessage,
        "Distance": FloatValue,
    },
    "ansys.api.edb.v1.PointTerminalService": {
        "Create": EDBObjMessage,
        "GetParameters": PointTermParamsMessage,
        "SetParameters": Empty,
    },
    "ansys.api.edb.v1.PolygonService": {
        "Create": EDBObjMessage,
        "GetPolygonData": PolygonDataMessage,
        "SetPolygonData": Empty,
    },
    "ansys.api.edb.v1.PolygonDataService": {
        "GetNormalizedPoints": PointsMessage,
        "IsCircle": BoolValue,
        "IsBox": BoolValue,
        "HasSelfIntersections": BoolValue,
        "RemoveSelfIntersections": PolygonDataListMessage,
        "IsConvex": BoolValue,
        "GetArea": FloatValue,
        "Transform": PolygonDataMessage,
        "GetBBox": BoxMessage,
        "GetStreamedBBox": BoxMessage,
        "GetConvexHull": PolygonDataMessage,
        "RemoveArcs": PolygonDataMessage,
        "Defeature": PolygonDataMessage,
        "GetBoundingCircle": CircleMessage,
        "IsInside": BoolValue,
        "CircleIntersectsPolygon": BoolValue,
        "GetIntersectionType": IntersectionTypeMessage,
        "GetClosestPoints": PointsMessage,
        "GetUnion": PolygonDataListMessage,
        "GetIntersection": PolygonDataListMessage,
        "Subtract": PolygonDataListMessage,
        "Xor": PolygonDataListMessage,
        "Expand": PolygonDataListMessage,
        "Get2DAlphaShape": PolygonDataListMessage,
    },
    "ansys.api.edb.v1.PortPropertyService": {
        "Create": EDBObjMessage,
        "Clone": EDBObjMessage,
        "GetReferenceHeight": ValueMessage,
        "SetReferenceHeight": Empty,
        "GetReferenceSizeAuto": BoolValue,
        "SetReferenceSizeAuto": Empty,
        "GetReferenceSize": ValuePairMessage,
        "SetReferenceSize": Empty,
    },
    "ansys.api.edb.v1.PrimitiveService": {
        "GetPrimitiveType": PrimitiveTypeMessage,
        "AddVoid": Empty,
        "SetHfssProp": Empty,
        "GetLayer": EDBObjMessage,
        "SetLayer": Empty,
        "GetIsNegative": BoolValue,
        "SetIsNegative": Empty,
        "IsVoid": BoolValue,
        "HasVoids": BoolValue,
        "Voids": EDBObjCollectionMessage,
        "StreamVoids": EDBObjCollectionMessage,
        "GetOwner": EDBObjMessage,
        "IsParameterized": BoolValue,
        "GetHfssProp": GetPrimitiveHfssPropMessage,
        "RemoveHfssProp": Empty,
        "IsZonePrimitive": BoolValue,
        "MakeZonePrimitive": Empty,
    },
    "ansys.api.edb.v1.PrimitiveInstanceCollectionService": {
        "Create": EDBObjMessage,
        "GetGeometry": PolygonDataMessage,
        "SetGeometry": Empty,
        "GetPositions": PointsMessage,
        "SetPositions": Empty,
        "GetInstantiatedGeometry": PolygonDataListMessage,
        "Decompose": Empty,
    },
    "ansys.api.edb.v1.Q3DGeneralSettingsService": {
        "GetSolutionFrequency": StringValue,
        "SetSolutionFrequency": Empty,
        "GetDoDC": BoolValue,
        "SetDoDC": Empty,
        "GetDoDCResOnly": BoolValue,
        "SetDoDCResOnly": Empty,
        "GetDoCG": BoolValue,
        "SetDoCG": Empty,
        "GetDoAC": BoolValue,
        "SetDoAC": Empty,
        "GetSaveFields": BoolValue,
        "SetSaveFields": Empty,
    },
    "ansys.api.edb.v1.Q3DSettingsService": {
        "GetMaxPasses": UInt64Value,
        "SetMaxPasses": Empty,
        "GetMinPasses": UInt64Value,
        "SetMinPasses": Empty,
        "GetMinConvergedPasses": UInt64Value,
        "SetMinConvergedPasses": Empty,
        "GetPercentError": DoubleValue,
        "SetPercentError": Empty,
        "GetMaxRefinePerPass": DoubleValue,
        "SetMaxRefinePerPass": Empty,
    },
    "ansys.api.edb.v1.Q3DDCRLSettingsService": {
        "GetSolutionOrder": Q3DSolutionOrderMessage,
        "SetSolutionOrder": Empty,
    },
    "ansys.api.edb.v1.Q3DCGSettingsService": {
        "GetAutoIncrSolOrder": BoolValue,
        "SetAutoIncrSolOrder": Empty,
        "GetSolutionOrder": Q3DSolutionOrderMessage,
        "SetSolutionOrder": Empty,
        "GetSolverType": SolverTypeMessage,
        "SetSolverType": Empty,
        "GetCompressionTol": DoubleValue,
        "SetCompressionTol": Empty,
    },
    "ansys.api.edb.v1.Q3DAdvancedSettingsService": {
        "GetICModeAutoResolution": BoolValue,
        "SetICModeAutoResolution": Empty,
        "GetICModeLength": StringValue,
        "SetICModeLength": Empty,
    },
    "ansys.api.edb.v1.Q3DAdvancedMeshingSettingsService": {
        "GetLayerAlignment": StringValue,
        "SetLayerAlignment": Empty,
    },
    "ansys.api.edb.v1.RaptorXGeneralSettingsService": {
        "GetUseGoldEMSolver": BoolValue,
        "SetUseGoldEMSolver": Empty,
        "GetMaxFrequency": StringValue,
        "SetMaxFrequency": Empty,
        "GetGlobalTemperature": DoubleValue,
        "SetGlobalTemperature": Empty,
        "GetSaveNetlist": BoolValue,
        "SetSaveNetlist": Empty,
        "GetNetlistExportSpectre": BoolValue,
        "SetNetlistExportSpectre": Empty,
        "GetSaveRFM": BoolValue,
        "SetSaveRFM": Empty,
    },
    "ansys.api.edb.v1.RaptorXAdvancedSettingsService": {
        "GetUseMeshFrequency": BoolValue,
        "SetUseMeshFrequency": Empty,
        "GetMeshFrequency": StringValue,
        "SetMeshFrequency": Empty,
        "GetUseEdgeMesh": BoolValue,
        "SetUseEdgeMesh": Empty,
        "GetEdgeMesh": StringValue,
        "SetEdgeMesh": Empty,
        "GetUseCellsPerWavelength": BoolValue,
        "SetUseCellsPerWavelength": Empty,
        "GetCellsPerWavelength": UInt64Value,
        "SetCellsPerWavelength": Empty,
        "GetUsePlaneProjectionFactor": BoolValue,
        "SetUsePlaneProjectionFactor": Empty,
        "GetPlaneProjectionFactor": DoubleValue,
        "SetPlaneProjectionFactor": Empty,
        "GetUseRelaxedZAxis": BoolValue,
        "SetUseRelaxedZAxis": Empty,
        "GetUseEliminateSlitPerHoles": BoolValue,
        "SetUseEliminateSlitPerHoles": Empty,
        "GetEliminateSlitPerHoles": DoubleValue,
        "SetEliminateSlitPerHoles": Empty,
        "GetUseAutoRemovalSliverPoly": BoolValue,
        "SetUseAutoRemovalSliverPoly": Empty,
        "GetAutoRemovalSliverPoly": DoubleValue,
        "SetAutoRemovalSliverPoly": Empty,
        "GetUseAccelerateViaExtraction": BoolValue,
        "SetUseAccelerateViaExtraction": Empty,
        "GetUseEnableSubstrateNetworkExtraction": BoolValue,
        "SetUseEnableSubstrateNetworkExtraction": Empty,
        "GetUseLDE": BoolValue,
        "SetUseLDE": Empty,
        "GetUseExtractFloatingMetalsDummy": BoolValue,
        "SetUseExtractFloatingMetalsDummy": Empty,
        "GetUseExtractFloatingMetalsFloating": BoolValue,
        "SetUseExtractFloatingMetalsFloating": Empty,
        "GetUseEnableEtchTransform": BoolValue,
        "SetUseEnableEtchTransform": Empty,
        "GetUseEnableHybridExtraction": BoolValue,
        "SetUseEnableHybridExtraction": Empty,
        "GetUseEnableAdvancedCapEffects": BoolValue,
        "SetUseEnableAdvancedCapEffects": Empty,
        "GetUseOverrideShrinkFac": BoolValue,
        "SetUseOverrideShrinkFac": Empty,
        "GetOverrideShrinkFac": DoubleValue,
        "SetOverrideShrinkFac": Empty,
        "GetAdvancedOptions": RaptorXSimSettingsOptionsMessage,
        "SetAdvancedOptions": Empty,
        "GetNetSettingsOptions": RaptorXSimSettingsOptionsMessage,
        "SetNetSettingsOptions": Empty,
    },
    "ansys.api.edb.v1.RectangleService": {
        "Create": EDBObjMessage,
        "GetParameters": RectangleParametersMessage,
        "SetParameters": Empty,
        "Render": PolygonDataMessage,
        "GetPolygonData": PolygonDataMessage,
    },
    "ansys.api.edb.v1.RLCComponentPropertyService": {
        "Create": EDBObjMessage,
        "GetEnabled": BoolValue,
        "SetEnabled": Empty,
    },
    "ansys.api.edb.v1.RTreeService": {
        "Create": EDBObjMessage,
        "GetExtent": BoxMessage,
        "InsertIntObject": Empty,
        "DeleteIntObject": Empty,
        "Empty": BoolValue,
        "Search": IntegersPropertyMessage,
        "NearestNeighbor": NearestNeighborResponseMessage,
        "TouchingGeometry": IntegersPropertyMessage,
        "ConnectedGeometry": IntegersPropertyMessage,
        "GetConnectedGeometrySets": GeometrySetsMessage,
        "IncrementVisit": Empty,
        "IsVisited": BoolValue,
        "Visit": Empty,
        "GetVisit": Int64Value,
    },
    "ansys.api.edb.v1.SimulationSettingsService": {"GetEnabled": BoolValue, "SetEnabled": Empty},
    "ansys.api.edb.v1.SettingsOptionsService": {
        "GetDoLamdaRefineFlag": BoolValue,
        "SetDoLamdaRefineFlag": Empty,
        "GetLamdaTarget": DoubleValue,
        "SetLamdaTarget": Empty,
        "GetMeshSizefactor": DoubleValue,
        "SetMeshSizefactor": Empty,
        "GetUseDefaultLamda": BoolValue,
        "SetUseDefaultLamda": Empty,
    },
    "ansys.api.edb.v1.AdvancedSettingsService": {
        "GetUnionPolygons": BoolValue,
        "SetUnionPolygons": Empty,
        "GetRemoveFloatingGeometry": BoolValue,
        "SetRemoveFloatingGeometry": Empty,
        "GetHealingOption": Int64Value,
        "SetHealingOption": Empty,
        "GetSmallVoidArea": DoubleValue,
        "SetSmallVoidArea": Empty,
        "GetUseDefeature": BoolValue,
        "SetUseDefeature": Empty,
        "GetUseDefeatureAbsoluteLength": BoolValue,
        "SetUseDefeatureAbsoluteLength": Empty,
        "GetDefeatureAbsoluteLength": StringValue,
        "SetDefeatureAbsoluteLength": Empty,
        "GetDefeatureRatio": DoubleValue,
        "SetDefeatureRatio": Empty,
        "GetViaModelType": ViaModelTypeMessage,
        "SetViaModelType": Empty,
        "GetNumViaSides": UInt64Value,
        "SetNumViaSides": Empty,
        "GetViaDensity": DoubleValue,
        "SetViaDensity": Empty,
        "GetViaMaterial": StringValue,
        "SetViaMaterial": Empty,
        "GetMeshForViaPlating": BoolValue,
        "SetMeshForViaPlating": Empty,
        "GetModelType": DefeatureModelTypeMessage,
        "SetModelType": Empty,
    },
    "ansys.api.edb.v1.AdvancedMeshingSettingsService": {
        "GetArcStepSize": StringValue,
        "SetArcStepSize": Empty,
        "GetCircleStartAzimuth": StringValue,
        "SetCircleStartAzimuth": Empty,
        "GetMaxNumArcPoints": UInt64Value,
        "SetMaxNumArcPoints": Empty,
        "GetUseArcChordErrorApprox": BoolValue,
        "SetUseArcChordErrorApprox": Empty,
        "GetArcChordErrorApprox": StringValue,
        "SetArcChordErrorApprox": Empty,
    },
    "ansys.api.edb.v1.SolverSettingsService": {
        "GetThinSignalLayerThreshold": StringValue,
        "SetThinSignalLayerThreshold": Empty,
        "GetThinDielectricLayerThreshold": StringValue,
        "SetThinDielectricLayerThreshold": Empty,
    },
    "ansys.api.edb.v1.SimulationSetupService": {
        "Create": EDBObjMessage,
        "GetName": StringValue,
        "SetName": Empty,
        "GetPosition": UInt64Value,
        "SetPosition": Empty,
        "GetSweepData": SweepDataListMessage,
        "SetSweepData": Empty,
        "GetType": SimulationSetupTypeMessage,
    },
    "ansys.api.edb.v1.SIWaveDCIRSimulationSettingsService": {
        "GetIcepakTempFile": StringValue,
        "SetIcepakTempFile": Empty,
        "GetSourceTermsToGround": SourceTermsToGroundMessage,
        "SetSourceTermsToGround": Empty,
        "GetExportDCThermalData": BoolValue,
        "SetExportDCThermalData": Empty,
        "GetImportThermalData": BoolValue,
        "SetImportThermalData": Empty,
        "GetFullDCReportPath": StringValue,
        "SetFullDCReportPath": Empty,
        "GetViaReportPath": StringValue,
        "SetViaReportPath": Empty,
        "GetPerPinResPath": StringValue,
        "SetPerPinResPath": Empty,
        "GetDCReportConfigFile": StringValue,
        "SetDCReportConfigFile": Empty,
        "GetDCReportShowActiveDevices": BoolValue,
        "SetDCReportShowActiveDevices": Empty,
        "GetPerPinUsePinFormat": BoolValue,
        "SetPerPinUsePinFormat": Empty,
        "GetUseLoopResForPerPin": BoolValue,
        "SetUseLoopResForPerPin": Empty,
    },
    "ansys.api.edb.v1.SIWavePSIGeneralSettingsService": {
        "GetPISliderPos": SIWavePSISimulationPreferenceMessage,
        "SetPISliderPos": Empty,
        "GetSIWavePSIModelType": SIWavePSIModelTypeMessage,
        "SetSIWavePSIModelType": Empty,
        "GetMinPlaneAreaToMesh": StringValue,
        "SetMinPlaneAreaToMesh": Empty,
        "GetMinVoidAreaToMesh": StringValue,
        "SetMinVoidAreaToMesh": Empty,
        "GetSnapLengthThreshold": StringValue,
        "SetSnapLengthThreshold": Empty,
        "GetIncludeEnhancedBondWireModeling": BoolValue,
        "SetIncludeEnhancedBondWireModeling": Empty,
        "GetSurfaceRoughnessModel": SurfaceRoughnessModelMessage,
        "SetSurfaceRoughnessModel": Empty,
        "GetRMSSurfaceRoughness": StringValue,
        "SetRMSSurfaceRoughness": Empty,
        "GetPerformERC": BoolValue,
        "SetPerformERC": Empty,
    },
    "ansys.api.edb.v1.SIWavePSINetProcessingSettingsService": {
        "GetAutoSelectNetsForSimulation": BoolValue,
        "SetAutoSelectNetsForSimulation": Empty,
        "GetIgnoreDummyNetsForSelectedNets": BoolValue,
        "SetIgnoreDummyNetsForSelectedNets": Empty,
        "GetIncludeNets": StringsMessage,
        "SetIncludeNets": Empty,
    },
    "ansys.api.edb.v1.SIWavePSIPowerGroundNetsService": {
        "GetImprovedLossModel": ImprovedLossModelMessage,
        "SetImprovedLossModel": Empty,
        "GetAutoDetectIgnoreSmallHolesMinDiameter": BoolValue,
        "SetAutoDetectIgnoreSmallHolesMinDiameter": Empty,
        "GetIgnoreSmallHolesMinDiameter": StringValue,
        "SetIgnoreSmallHolesMinDiameter": Empty,
    },
    "ansys.api.edb.v1.SIWavePSISignalNetsSettingsService": {
        "GetSignalNetsErrorTolerance": ErrorToleranceMessage,
        "SetSignalNetsErrorTolerance": Empty,
        "GetSignalNetsConductorModeling": ConductorModelingMessage,
        "SetSignalNetsConductorModeling": Empty,
        "GetSignalNetsIncludeImprovedLossHandling": BoolValue,
        "SetSignalNetsIncludeImprovedLossHandling": Empty,
        "GetSignalNetsIncludeImprovedDielectricFillRefinement": BoolValue,
        "SetSignalNetsIncludeImprovedDielectricFillRefinement": Empty,
    },
    "ansys.api.edb.v1.SIWaveGeneralSettingsService": {
        "GetUseSISettings": BoolValue,
        "SetUseSISettings": Empty,
        "GetUseCustomSettings": BoolValue,
        "SetUseCustomSettings": Empty,
        "GetSISliderPos": UInt64Value,
        "SetSISliderPos": Empty,
        "GetPISliderPos": UInt64Value,
        "SetPISliderPos": Empty,
    },
    "ansys.api.edb.v1.SIWaveAdvancedSettingsService": {
        "GetIncludeCoPlaneCoupling": BoolValue,
        "SetIncludeCoPlaneCoupling": Empty,
        "GetIncludeInterPlaneCoupling": BoolValue,
        "SetIncludeInterPlaneCoupling": Empty,
        "GetIncludeSplitPlaneCoupling": BoolValue,
        "SetIncludeSplitPlaneCoupling": Empty,
        "GetIncludeFringePlaneCoupling": BoolValue,
        "SetIncludeFringePlaneCoupling": Empty,
        "GetIncludeTracePlaneCoupling": BoolValue,
        "SetIncludeTracePlaneCoupling": Empty,
        "GetCrossTalkThreshold": StringValue,
        "SetCrossTalkThreshold": Empty,
        "GetMaxCoupledLines": UInt64Value,
        "SetMaxCoupledLines": Empty,
        "GetMinVoidArea": StringValue,
        "SetMinVoidArea": Empty,
        "GetMinPadAreaToMesh": StringValue,
        "SetMinPadAreaToMesh": Empty,
        "GetMinPlaneAreaToMesh": StringValue,
        "SetMinPlaneAreaToMesh": Empty,
        "GetSnapLengthThreshold": StringValue,
        "SetSnapLengthThreshold": Empty,
        "GetMeshAutomatic": BoolValue,
        "SetMeshAutomatic": Empty,
        "GetMeshFrequency": StringValue,
        "SetMeshFrequency": Empty,
        "GetAcDcMergeMode": ACDCMergeModeMessage,
        "SetAcDcMergeMode": Empty,
        "Get3DReturnCurrentDistribution": BoolValue,
        "Set3DReturnCurrentDistribution": Empty,
        "GetIncludeVISources": BoolValue,
        "SetIncludeVISources": Empty,
        "GetIncludeInfGnd": BoolValue,
        "SetIncludeInfGnd": Empty,
        "GetInfGndLocation": StringValue,
        "SetInfGndLocation": Empty,
        "GetPerformERC": BoolValue,
        "SetPerformERC": Empty,
        "GetIgnoreNonFunctionalPads": BoolValue,
        "SetIgnoreNonFunctionalPads": Empty,
    },
    "ansys.api.edb.v1.SIWaveDCSettingsService": {
        "GetUseDCCustomSettings": BoolValue,
        "SetUseDCCustomSettings": Empty,
        "GetComputeInductance": BoolValue,
        "SetComputeInductance": Empty,
        "GetPlotJV": BoolValue,
        "SetPlotJV": Empty,
        "GetContactRadius": StringValue,
        "SetContactRadius": Empty,
        "GetDCSliderPos": UInt64Value,
        "SetDCSliderPos": Empty,
    },
    "ansys.api.edb.v1.SIWaveDCAdvancedSettingsService": {
        "GetDCMinPlaneAreaToMesh": StringValue,
        "SetDCMinPlaneAreaToMesh": Empty,
        "GetDCMinVoidAreaToMesh": StringValue,
        "SetDCMinVoidAreaToMesh": Empty,
        "GetMaxInitMeshEdgeLength": StringValue,
        "SetMaxInitMeshEdgeLength": Empty,
        "GetPerformAdaptiveRefinement": BoolValue,
        "SetPerformAdaptiveRefinement": Empty,
        "GetMaxNumPasses": UInt64Value,
        "SetMaxNumPasses": Empty,
        "GetMinNumPasses": UInt64Value,
        "SetMinNumPasses": Empty,
        "GetPercentLocalRefinement": UInt64Value,
        "SetPercentLocalRefinement": Empty,
        "GetEnergyError": DoubleValue,
        "SetEnergyError": Empty,
        "GetMeshBws": BoolValue,
        "SetMeshBws": Empty,
        "GetRefineBws": BoolValue,
        "SetRefineBws": Empty,
        "GetMeshVias": BoolValue,
        "SetMeshVias": Empty,
        "GetRefineVias": BoolValue,
        "SetRefineVias": Empty,
        "GetNumBwSides": UInt64Value,
        "SetNumBwSides": Empty,
        "GetNumViaSides": UInt64Value,
        "SetNumViaSides": Empty,
    },
    "ansys.api.edb.v1.SIWaveSParameterSettingsService": {
        "GetUseStateSpace": BoolValue,
        "SetUseStateSpace": Empty,
        "GetInterpolation": SParamInterpolationMessage,
        "SetInterpolation": Empty,
        "GetExtrapolation": SParamExtrapolationMessage,
        "SetExtrapolation": Empty,
        "GetDCBehavior": SParamDCBehaviorMessage,
        "SetDCBehavior": Empty,
    },
    "ansys.api.edb.v1.SolderBallPropertyService": {
        "Create": EDBObjMessage,
        "GetShape": PadstackDefDataSolderballShapeMessage,
        "SetShape": Empty,
        "GetPlacement": PadstackDefDataSolderballPlacementMessage,
        "SetPlacement": Empty,
        "GetDiameter": DiameterMessage,
        "SetDiameter": Empty,
        "GetHeight": ValueMessage,
        "SetHeight": Empty,
        "GetMaterialName": StringValue,
        "SetMaterialName": Empty,
        "UsesSolderball": BoolValue,
        "Clone": EDBObjMessage,
    },
    "ansys.api.edb.v1.SpiceModelService": {
        "Create": EDBObjMessage,
        "GetProperties": SpiceModelMessage,
        "SetModelPath": Empty,
        "SetModelName": Empty,
        "SetSubCkt": Empty,
        "AddTerminalPinPair": Empty,
        "RemoveTerminalPinPair": Empty,
        "GetTerminalPinPairs": StringMapMessage,
    },
    "ansys.api.edb.v1.StackupLayerService": {
        "Create": EDBObjMessage,
        "GetNegative": BoolValue,
        "SetNegative": Empty,
        "GetThickness": ValueMessage,
        "SetThickness": Empty,
        "GetLowerElevation": ValueMessage,
        "SetLowerElevation": Empty,
        "GetUpperElevation": ValueMessage,
        "GetMaterial": StringValue,
        "SetMaterial": Empty,
        "GetFillMaterial": StringValue,
        "SetFillMaterial": Empty,
        "SetRoughnessEnabled": Empty,
        "IsRoughnessEnabled": BoolValue,
        "GetRoughnessModel": RoughnessModelMessage,
        "SetRoughnessModel": Empty,
        "IsEtchFactorEnabled": BoolValue,
        "SetEtchFactorEnabled": Empty,
        "SetEtchFactor": Empty,
        "GetEtchFactor": ValueMessage,
        "SetEtchNetClass": Empty,
        "GetEtchNetClass": EtchNetClassMessage,
        "GetUseSolverProperties": BoolValue,
        "SetUseSolverProperties": Empty,
        "GetHFSSSolverProperties": HFSSSolverPropertiesMessage,
        "SetHFSSSolverProperties": Empty,
        "GetReferencingViaLayerIds": ViaLayerIdsMessage,
    },
    "ansys.api.edb.v1.Structure3DService": {
        "Create": EDBObjMessage,
        "GetMaterial": StringValue,
        "SetMaterial": Empty,
        "GetThickness": ValueMessage,
        "SetThickness": Empty,
        "GetMeshClosureProp": ClosureMessage,
        "SetMeshClosureProp": Empty,
    },
    "ansys.api.edb.v1.SParameterModelService": {
        "Create": EDBObjMessage,
        "GetProperties": SParameterModelMessage,
        "SetComponentModelName": Empty,
        "SetReferenceNet": Empty,
    },
    "ansys.api.edb.v1.TechnologyDefService": {
        "Create": EDBObjMessage,
        "FindByName": EDBObjMessage,
        "Delete": Empty,
        "GetTechFile": StringValue,
        "GetGFDFile": StringValue,
        "GetLayerFile": StringValue,
        "GetName": StringValue,
        "GetIsCreateBackplane": BoolValue,
    },
    "ansys.api.edb.v1.TerminalService": {
        "FindByName": EDBObjMessage,
        "GetParams": TermParamsMessage,
        "SetParams": Empty,
        "GetProductSolvers": TermProductSolversMessage,
        "SetProductSolverOptions": Empty,
    },
    "ansys.api.edb.v1.TerminalInstanceService": {
        "Create": EDBObjMessage,
        "GetOwningCellInstance": EDBObjMessage,
        "GetDefinitionTerminal": EDBObjMessage,
        "GetDefinitionTerminalName": StringValue,
    },
    "ansys.api.edb.v1.TerminalInstanceTerminalService": {
        "Create": EDBObjMessage,
        "GetTerminalInstance": EDBObjMessage,
        "SetTerminalInstance": Empty,
    },
    "ansys.api.edb.v1.TextService": {
        "Create": EDBObjMessage,
        "GetTextData": TextDataMessage,
        "SetTextData": Empty,
    },
    "ansys.api.edb.v1.TransformService": {
        "Rotate": PointMessage,
        "Create": EDBObjMessage,
        "GetScale": ValueMessage,
        "SetScale": Empty,
        "GetMirror": BoolValue,
        "SetMirror": Empty,
        "GetRotation": ValueMessage,
        "SetRotation": Empty,
        "GetOffsetX": ValueMessage,
        "SetOffsetX": Empty,
        "GetOffsetY": ValueMessage,
        "SetOffsetY": Empty,
        "TransformPlus": EDBObjMessage,
        "IsIdentity": BoolValue,
        "TransformPoint": PointMessage,
        "TransformPolygon": PolygonDataMessage,
    },
    "ansys.api.edb.v1.Transform3DService": {
        "CreateIdentity": EDBObjMessage,
        "CreateCopy": EDBObjMessage,
        "CreateOffset": EDBObjMessage,
        "CreateCenterScale": EDBObjMessage,
        "CreateRotationFromAngle": EDBObjMessage,
        "CreateRotationFromAxis": EDBObjMessage,
        "CreateRotationFromAxisAndAngle": EDBObjMessage,
        "CreateRotationFromToAxis": EDBObjMessage,
        "CreateTransform2D": EDBObjMessage,
        "OperatorPlus": EDBObjMessage,
        "TransformPoint": CPos3DMessage,
        "GetZYXRotation": CPos3DMessage,
        "GetAxis": CPos3DTripleMessage,
        "Transpose": Empty,
        "Invert": Empty,
        "IsIdentity": BoolValue,
        "IsEqual": BoolValue,
        "GetScaling": CPos3DMessage,
        "GetShift": CPos3DMessage,
        "SetMatrix": Empty,
        "GetMatrix": DoublesMessage,
    },
    "ansys.api.edb.v1.ValueService": {
        "CreateValue": ValueMessage,
        "GetDouble": DoubleValue,
        "GetComplex": ComplexMessage,
    },
    "ansys.api.edb.v1.VariableServerService": {
        "AddVariable": Empty,
        "AddMenuVariable": Empty,
        "DeleteVariable": Empty,
        "SetVariableValue": Empty,
        "GetVariableValue": ValueMessage,
        "IsParameter": BoolValue,
        "GetAllVariableNames": GetVariableNamesMessage,
        "GetVariableDesc": StringValue,
        "SetVariableDesc": Empty,
    },
    "ansys.api.edb.v1.ViaGroupService": {
        "CreateWithPrimitives": EDBObjCollectionMessage,
        "CreateWithOutline": EDBObjMessage,
        "FindByName": EDBObjMessage,
        "GetOutline": PolygonDataMessage,
        "GetConductorPercentage": DoubleValue,
        "IsPersistent": BoolValue,
    },
    "ansys.api.edb.v1.ViaLayerService": {
        "Create": EDBObjMessage,
        "GetRefLayerName": StringValue,
        "SetRefLayer": Empty,
        "GetIsTSV": BoolValue,
        "SetIsTSV": Empty,
        "AddOxideLayers": Empty,
        "RemoveOxideLayers": Empty,
        "GetNumOxideLayers": UInt64Value,
        "GetOxideLayers": OxideLayersMessage,
        "SetOxideLayerData": Empty,
    },
    "ansys.api.edb.v1.VoltageRegulatorService": {
        "Create": EDBObjMessage,
        "GetName": StringValue,
        "SetName": Empty,
        "IsActive": BoolValue,
        "SetIsActive": Empty,
        "GetVoltage": ValueMessage,
        "SetVoltage": Empty,
        "GetLoadRegulationCurrent": ValueMessage,
        "SetLoadRegulationCurrent": Empty,
        "GetLoadRegulationPercent": ValueMessage,
        "SetLoadRegulationPercent": Empty,
        "GetPosRemoteSensePin": EDBObjMessage,
        "SetPosRemoteSensePin": Empty,
        "GetNegRemoteSensePin": EDBObjMessage,
        "SetNegRemoteSensePin": Empty,
        "GetNPowerModules": Int64Value,
        "GetNActivePowerModules": Int64Value,
        "GetPowerModule": PowerModuleMessage,
        "GetAllPowerModules": PowerModulesMessage,
        "AddPowerModule": Empty,
        "RemovePowerModule": Empty,
        "AddPowerModules": Empty,
        "RemovePowerModules": Empty,
        "RemoveAllPowerModules": Empty,
    },
}


def get_rpc_response_type(service_name, rpc_name):
    """Get the response type for the provided service and RPC names."""
    service_map = rpc_response_map.get(service_name)
    return service_map.get(rpc_name) if service_map is not None else None



from contextlib import closing
from pathlib import Path
import socket

import settings

from ansys.edb.core.database import Database
import ansys.edb.core.session as session


def test_launch_session_when_default_port_in_use(new_database_path: Path):
    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as sock:
        sock.bind(("127.0.0.1", 50051))
        sess: session._Session = session.launch_session(settings.server_exe_dir())
        try:
            assert isinstance(sess.port_num, int) and sess.port_num != 50051
            with closing(Database.create(new_database_path)) as db:
                assert not db.is_null
        finally:
            sess.disconnect()


def test_session_when_default_port_in_use(new_database_path: Path):
    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as sock:
        sock.bind(("127.0.0.1", 50051))
        with session.session(settings.server_exe_dir()):
            sess: session._Session = session.MOD.current_session
            assert isinstance(sess.port_num, int) and sess.port_num != 50051
            with closing(Database.create(new_database_path)) as db:
                assert not db.is_null



import pytest

from ansys.edb.core.layout.cell import Cell
from ansys.edb.core.simulation_setup.simulation_setup import Distribution, FrequencyData, SweepData
from ansys.edb.core.simulation_setup.siwave_simulation_setup import SIWaveSimulationSetup


@pytest.mark.parametrize(
    "frequency_data",
    [
        FrequencyData(Distribution.LINC, "0GHz", "1GHz", "1001"),
        [
            FrequencyData(Distribution.LINC, "0GHz", "1GHz", "1001"),
            FrequencyData(Distribution.DEC, "1kHz", "1GHz", "20"),
        ],
    ],
)
def test_create_siwave_simulation_setup(
    circuit_cell: Cell, frequency_data: FrequencyData | list[FrequencyData]
):
    setup = SIWaveSimulationSetup.create(circuit_cell, "Setup1")
    sweep_data = SweepData("Sweep1", frequency_data)
    assert isinstance(sweep_data.frequency_data, type(frequency_data))
    setup.sweep_data = [sweep_data]
    assert isinstance(setup.sweep_data[0].frequency_data, type(frequency_data))
    if isinstance(frequency_data, list):
        for i, fd in enumerate(frequency_data):
            _assert_frequency_data_are_equal(fd, setup.sweep_data[0].frequency_data[i])
    else:
        _assert_frequency_data_are_equal(frequency_data, setup.sweep_data[0].frequency_data)


def _assert_frequency_data_are_equal(fd1: FrequencyData, fd2: FrequencyData):
    assert fd1.distribution == fd2.distribution
    assert fd1.start_f == fd2.start_f
    assert fd1.end_f == fd2.end_f
    assert fd1.step == fd2.step



import settings

from ansys.edb.core.database import Database
from ansys.edb.core.layer.layer import LayerType
from ansys.edb.core.layer.stackup_layer import StackupLayer
from ansys.edb.core.layout.cell import Cell, CellType
from ansys.edb.core.net.net import Net
from ansys.edb.core.primitive.circle import Circle
from ansys.edb.core.session import launch_session
from ansys.edb.core.utility.value import Value


def test_value():
    session = launch_session(settings.server_exe_dir(), 50051)

    db = Database.create(settings.temp_dir())
    db.add_variable("$green", 0.5)

    print(db.get_all_variable_names())

    cell = Cell.create(db, CellType.CIRCUIT_CELL, "my_cell")
    layout = cell.layout
    net = Net.create(layout, "net_1")
    lc = layout.layer_collection
    layer = StackupLayer.create("layer_1", LayerType.SIGNAL_LAYER, 0.0001, 0, "copper")
    lc.add_layers([layer])
    layer2 = lc.find_by_name("layer_1")

    layout.layer_collection = lc

    param = Value(33.1)
    cell.add_variable("blue1", param)
    cell.add_variable("blue2", "25mm")
    vv = cell.create_value("blue1 + blue2")
    print(vv.double)

    layout.add_variable("c_blue", param)

    print(cell.get_all_variable_names())

    vv = cell.create_value("c_blue + 10mm")
    print("vv = " + str(vv))
    print("value of vv is ", vv.double)
    print("complex of vv is ", vv.complex)

    print("start circle parameter testing")
    circ = Circle.create(layout, layer2, net, 1.335, "2.7mm", "c_blue+$green")

    params = circ.get_parameters()
    print(
        "After creation: params[0] = "
        + str(params[0])
        + " params[1] = "
        + str(params[1])
        + " params[2] = "
        + str(params[2])
    )
    print(
        "Same, but doubles: params[0] = "
        + str(params[0].double)
        + " params[1] = "
        + str(params[1].double)
        + " params[2] = "
        + str(params[2].double)
    )
    print(
        "Same, but is_parametric: params[0] = "
        + str(params[0].is_parametric)
        + " params[1] = "
        + str(params[1].is_parametric)
        + " params[2] = "
        + str(params[2].is_parametric)
    )

    circ.set_parameters(4, "0.355MHz", "c_blue-0.009")

    params = circ.get_parameters()
    print(
        "After setting: params[0] = "
        + str(params[0])
        + " params[1] = "
        + str(params[1])
        + " params[2] = "
        + str(params[2])
    )
    print(
        "Same, but doubles: params[0] = "
        + str(params[0].double)
        + " params[1] = "
        + str(params[1].double)
        + " params[2] = "
        + str(params[2].double)
    )

    session.disconnect()


if __name__ == "__main__":
    test_value()



def test_get_database(circuit_cell, new_database):
    cell_db = circuit_cell.database
    assert not cell_db.is_null
    assert cell_db.id == new_database.id


def test_get_layout(circuit_cell):
    cell_lyt = circuit_cell.layout
    assert not cell_lyt.is_null
    assert circuit_cell.id == cell_lyt.cell.id



"""Scratch tests for the rpc_executor C++ extension module.

Run with::

    pytest tests/e2e/scratch/test_rpc_executor.py -v

Tests that exercise the real EDB_RPC_Services library require the
``ANSYSEM_EDB_EXE_DIR`` environment variable to point to an Ansys EM install
directory containing ``EDB_RPC_Services.dll`` (Windows) or
``libEDB_RPC_Services.so`` (Linux).  Those tests are skipped automatically
when the variable is unset.

Tests that verify failure-mode behaviour (bad path, pre-initialize call) run
unconditionally and do not need a real EDB install.
"""

import sys
#sys.path.append(r"/srv/dmiller/installations/ansys_inc/v261_02_02_2026/AnsysEM")

import os

import pytest
import rpc_executor

# ---------------------------------------------------------------------------
# Helpers
# ---------------------------------------------------------------------------

EDB_EXE_DIR = os.environ.get("ANSYSEM_ROOT261")

requires_install = pytest.mark.skipif(
    not EDB_EXE_DIR,
    reason="ANSYSEM_EDB_EXE_DIR not set — skipping tests that require a real EDB install",
)

# ---------------------------------------------------------------------------
# Tests that do NOT need a real EDB install
# ---------------------------------------------------------------------------


def test_module_exposes_expected_callables():
    """The compiled extension must expose initialize() and execute_rpc()."""
    assert callable(rpc_executor.initialize)
    assert callable(rpc_executor.execute_rpc)


def test_initialize_with_nonexistent_path_returns_false():
    """initialize() must return False for a path that does not exist."""
    result = rpc_executor.initialize("/does/not/exist/at/all")
    assert result is False, "Expected False for a non-existent directory"


def test_initialize_with_empty_string_returns_false():
    """initialize() must return False for an empty path."""
    result = rpc_executor.initialize("")
    assert result is False, "Expected False for an empty directory path"


def test_execute_rpc_before_initialize_raises_runtime_error():
    """execute_rpc() must raise RuntimeError when Initialize() was never called.

    This test is only meaningful when run in isolation before any successful
    initialize() call.  When ANSYSEM_EDB_EXE_DIR is set, the @requires_install
    tests that run before this one will have already called initialize(), so
    g_plugin will be non-null and this test is skipped automatically.
    """
    if EDB_EXE_DIR:
        pytest.skip(
            "ANSYSEM_EDB_EXE_DIR is set; initialize() will have already been "
            "called by an earlier test — g_plugin is non-null.  Run this test "
            "file in isolation (without ANSYSEM_EDB_EXE_DIR) to exercise the "
            "uninitialized guard."
        )
    with pytest.raises(RuntimeError, match="not initialized"):
        rpc_executor.execute_rpc("SomeService", "SomeMethod", b"")


# ---------------------------------------------------------------------------
# Tests that require a real EDB install
# ---------------------------------------------------------------------------


@requires_install
def test_initialize_with_valid_dir_returns_true():
    """initialize() must return True for a valid Ansys EM install directory."""
    ok = rpc_executor.initialize(EDB_EXE_DIR)
    assert ok, f"rpc_executor.initialize({EDB_EXE_DIR!r}) returned False"


@requires_install
def test_initialize_is_noop_after_first_success():
    """After the first successful initialize(), further calls are a no-op.

    The second call uses a deliberately wrong path; if Initialize() re-ran it
    would return False.  It must return True because g_plugin is already set.
    """
    rpc_executor.initialize(EDB_EXE_DIR)
    ok = rpc_executor.initialize("/does/not/exist")
    assert ok, "Second call to initialize() should be a no-op and return True"


@requires_install
def test_execute_rpc_returns_three_tuple():
    """execute_rpc() must return a (bool, bytes, str) 3-tuple."""
    rpc_executor.initialize(EDB_EXE_DIR)
    result = rpc_executor.execute_rpc("EDBServer", "GetVersion", b"")
    assert isinstance(result, tuple), "execute_rpc() did not return a tuple"
    assert len(result) == 3, f"Expected 3-tuple, got length {len(result)}"
    ok, response, error = result
    assert isinstance(ok, bool), f"result[0] should be bool, got {type(ok)}"
    assert isinstance(response, bytes), f"result[1] should be bytes, got {type(response)}"
    assert isinstance(error, str), f"result[2] should be str, got {type(error)}"


@requires_install
def test_execute_rpc_serialized_request_must_be_bytes():
    """execute_rpc() must reject a non-bytes serialized_request argument."""
    rpc_executor.initialize(EDB_EXE_DIR)
    with pytest.raises(TypeError):
        rpc_executor.execute_rpc("EDBServer", "GetVersion", "not-bytes")

@requires_install
def test_load_edb():
    from ansys.edb.core.session import launch_local_session
    import os
    from ansys.edb.core.database import Database
    launch_local_session(EDB_EXE_DIR)
    #db = Database.open(r"/srv/dmiller/installations/ansys_inc/v261_02_02_2026/AnsysEM/Examples/HFSS 3D Layout/Signal Integrity/Diff_Via.aedb", True)


@requires_install
def test_coexistence_with_cpp_extension_libraries():
    """Load numpy, scipy, pandas, grpc and protobuf (all C++-backed) alongside
    rpc_executor and then open an EDB database to confirm that the .pth preload
    hook correctly establishes a compatible libstdc++.so.6 before any of these
    extensions can claim the system version.

    Each library is exercised with a minimal but real operation so we can be
    confident its own C extension initialised properly, not just that the module
    object imported without error.
    """
    # --- numpy ------------------------------------------------------------------
    import numpy as np
    arr = np.array([1.0, 2.0, 3.0])
    assert np.sum(arr) == 6.0, "numpy sum failed"
    assert arr.dtype == np.float64

    # --- scipy ------------------------------------------------------------------
    import scipy
    from scipy.special import expit  # logistic sigmoid — exercises Cython layer
    result = expit(0.0)
    assert abs(result - 0.5) < 1e-9, f"scipy.special.expit(0) = {result}, expected 0.5"

    # --- pandas -----------------------------------------------------------------
    import pandas as pd
    df = pd.DataFrame({"x": [1, 2, 3], "y": [4, 5, 6]})
    assert list(df.columns) == ["x", "y"]
    assert df["x"].sum() == 6

    # --- grpc (cygrpc Cython extension) ----------------------------------------
    import grpc
    channel = grpc.insecure_channel("localhost:1")  # not connected, just creates object
    assert channel is not None
    channel.close()

    # --- protobuf (upb C extension) --------------------------------------------
    from google.protobuf import descriptor_pool
    pool = descriptor_pool.Default()
    assert pool is not None

    # --- EDB database open (the core integration check) -----------------------
    from ansys.edb.core.session import launch_local_session
    from ansys.edb.core.database import Database
    launch_local_session(EDB_EXE_DIR)
    db = Database.open(
        r"/srv/dmiller/installations/ansys_inc/v261_02_02_2026/AnsysEM/Examples/HFSS 3D Layout/Signal Integrity/Diff_Via.aedb",
        True,
    )
    assert db is not None, "Database.open returned None"


from ansys.edb.core import __version__


def test_pkg_version():
    try:
        import importlib.metadata as importlib_metadata
    except ModuleNotFoundError:  # pragma: no cover
        import importlib_metadata

    # Read from the pyproject.toml
    # major, minor, patch
    read_version = importlib_metadata.version("ansys-edb-core")

    assert __version__ == read_version



"""A pre-commit hook script for cleaning Jupyter notebooks by resetting execution counts, removing outputs, and removing metadata."""  # noqa: E501

import argparse
import json
from pathlib import Path
import sys


def _clean_notebook(path: Path, leave_outputs: bool):
    """Clean an ipynb file and return True if it was modified, None on error, otherwise False."""
    modified = False
    try:
        with open(path) as input_file:
            data = json.load(input_file)
        execution_order = 1
        if "metadata" in data and data["metadata"]:
            data["metadata"] = {}
            modified = True
        for cell in data["cells"]:
            if "execution_count" in cell:
                if leave_outputs:
                    if cell["execution_count"] != execution_order:
                        cell["execution_count"] = execution_order
                        modified = True
                    execution_order += 1
                else:
                    if cell["execution_count"] is not None:
                        cell["execution_count"] = None
                        modified = True
            if "metadata" in cell and cell["metadata"]:
                cell["metadata"] = {}
                modified = True
            if not leave_outputs and "outputs" in cell and cell["outputs"]:
                cell["outputs"] = []
                modified = True
        if modified:
            print(path)
            with open(path, "w") as output_file:
                json.dump(data, output_file, indent=1)
    except Exception as e:
        print("Unexpected exception: {}".format(e))
        return None
    return modified


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--leave-outputs", action="store_true", default=False, help="leave cell outputs as-is"
    )
    parser.add_argument("files", help="ipynb files to process", nargs=argparse.REMAINDER)
    arguments = parser.parse_args()
    files_modified = False
    for file in arguments.files:
        path = Path(file)
        if not path.is_file() or path.suffix != ".ipynb":
            continue
        result = _clean_notebook(path, arguments.leave_outputs)
        if result is None:
            print("_clean_notebook failed on {}".format(path))
            sys.exit(2)
        files_modified = result

    sys.exit(int(files_modified))



import os
import platform
import tempfile

import settings

from ansys.edb.core.database import Database
from ansys.edb.core.definition.material_def import MaterialDef, MaterialProperty
from ansys.edb.core.geometry.polygon_data import PolygonData
from ansys.edb.core.hierarchy.via_group import ViaGroup
from ansys.edb.core.layer.layer import LayerType
from ansys.edb.core.layer.layer_collection import LayerCollection, LayerCollectionMode
from ansys.edb.core.layer.stackup_layer import StackupLayer
from ansys.edb.core.layer.via_layer import ViaLayer
from ansys.edb.core.layout.cell import Cell, CellType
from ansys.edb.core.net.net import Net
from ansys.edb.core.primitive.path import Path, PathCornerType, PathEndCapType
from ansys.edb.core.primitive.polygon import Polygon
from ansys.edb.core.primitive.rectangle import Rectangle, RectangleRepresentationType
from ansys.edb.core.session import session
from ansys.edb.core.simulation_setup.adaptive_solutions import SingleFrequencyAdaptiveSolution
from ansys.edb.core.simulation_setup.hfss_simulation_setup import HfssSimulationSetup
from ansys.edb.core.simulation_setup.mesh_operation import SkinDepthMeshOperation
from ansys.edb.core.simulation_setup.simulation_setup import Distribution, FrequencyData, SweepData
from ansys.edb.core.terminal.point_terminal import PointTerminal

# Wrapper class over Database
# This will ensure clean entry and exit from database
from ansys.edb.core.utility.hfss_extent_info import HfssExtentInfo, HFSSExtentInfoType


class TDatabase:
    def __init__(self, path: str):
        rootdir = tempfile.gettempdir()
        rootdir = os.path.join(rootdir, "pyedb")
        rootdir = os.path.join(rootdir, platform.python_version())
        path = os.path.join(rootdir, path)
        print("deleting old database at", path)
        Database.delete(path)

        print("creating database at", path)
        self.db = Database.create(path)

    def __enter__(self):
        print("__enter__")
        pass

    def __exit__(self, exc_type, exc_val, exc_tb):
        print("__exit__")
        self.db.save()
        self.db.close()

    def database(self):
        return self.db


class BaseExample:
    def database(self, path: str) -> TDatabase:
        db = TDatabase(path)
        self.db = db.database()
        print("creating default cell")
        self.cell = Cell.create(self.db, CellType.CIRCUIT_CELL, "EMDesign1")
        print("assigning default layout")
        self.layout = self.cell.layout
        return db

    def __init__(self):
        self.db = None
        self.cell = None
        self.layout = None
        self.nets = {}

    def run(self):
        pass

    def create_material(self, name, properties):
        mat = MaterialDef.create(self.db, name)
        for key, value in properties.items():
            MaterialDef.set_property(mat, key, value)
        return mat

    def net(self, name):
        if name not in self.nets:
            self.nets[name] = Net.create(self.layout, name)

        return self.nets[name]

    def create_rectangle(self, layer_name, net_name, llx, lly, urx, ury):
        return Rectangle.create(
            self.layout,
            layer_name,
            self.net(net_name),
            RectangleRepresentationType.LOWER_LEFT_UPPER_RIGHT,
            llx,
            lly,
            urx,
            ury,
            0.0,
            0.0,
        )

    def create_polygon(self, layer_name, net_name, vertices, holes=[]):
        polygon_data = PolygonData(vertices, closed=True)
        polygon = Polygon.create(self.layout, layer_name, self.net(net_name), polygon_data)
        if len(holes):
            hole_polygon = self.create_polygon(layer_name, net_name, holes)
            polygon.add_void(hole_polygon)

        return polygon

    def create_path(self, layer_name, net_name, width, vertices):
        polygon_data = PolygonData(vertices, closed=True)
        return Path.create(
            self.layout,
            layer_name,
            self.net(net_name),
            width,
            PathEndCapType.FLAT,
            PathEndCapType.FLAT,
            PathCornerType.SHARP,
            polygon_data,
        )

    def create_point_terminal(self, net_name, name, point, layer_name):
        return PointTerminal.create(self.layout, self.net(net_name), layer_name, name, point)

    def set_hfss_extents(self, extents):
        self.cell.set_hfss_extent_info(extents)


def um(val, val2=None):
    if val2:
        return um(val), um(val2)
    else:
        return val * 1e-6


materials = [
    ("IMD1", (0, 3.56, 1, 0)),
    ("IMD7", (0, 4.299, 1, 0)),
    ("IMD9", (0, 4.285, 1, 0)),
    ("MetalRDL", (33300000, 1, 1, 0)),
    ("MetalU", (53475900, 1, 1, 0)),
    ("Metalx", (24690000, 1, 1, 0)),
    ("MetalZ", (55586400, 1, 1, 0)),
    ("My_Sub1", (10, 11.8, 1, 0)),
    ("Passivation", (0, 4.472, 1, 0)),
    ("SolderMask", (0, 3.1, 1, 0.035)),
    ("Underfill", (0, 3.7, 1, 0)),
    ("vacuum", (0, 1, 1, 0)),
    ("VIAuz", (3222000, 1, 1, 0)),
    ("VIAx-CO", (854198, 1, 1, 0)),
    ("VIAz", (3358800, 1, 1, 0)),
    ("copper", (5.8e8, 1, 0.999991, 0)),
]

dielectrics = [
    ("SolderMask", "SolderMask", 15),
    ("Underfill", "Underfill", 68),
    ("Passivation", "Passivation", 4.7),
    ("IMD9", "IMD9", 4.215),
    ("IMD7", "IMD7", 3.045),
    ("IMD1", "IMD1", 1.985),
    ("Sub", "My_Sub1", 100),
]

signals = [
    ("OVERPASS", "MetalRDL", 2.8, 109.245),
    ("VOVERPASS", "MetalRDL", 0.775, 108.47),
    ("M9", "MetalU", 3.5, 104.97),
    ("M8", "MetalZ", 0.85, 103.38),
    ("M7", "MetalZ", 0.85, 101.935),
    ("M6", "Metalx", 0.09, 101.25),
    ("M5", "Metalx", 0.09, 101.075),
    ("M4", "Metalx", 0.09, 100.9),
    ("M3", "Metalx", 0.09, 100.725),
    ("M2", "Metalx", 0.09, 100.55),
    ("M1", "Metalx", 0.09, 100.375),
    ("CO", "VIAx-CO", 0.375, 100),
    ("Ground", "copper", 0, 0),
]

vias = [
    ("V8", "M9", "M8", "VIAuz"),
    ("V7", "M8", "M7", "VIAz"),
    ("V6", "M7", "M6", "VIAz"),
    ("V5", "M6", "M5", "VIAx-CO"),
    ("V4", "M5", "M4", "VIAx-CO"),
    ("V3", "M4", "M3", "VIAx-CO"),
    ("V2", "M3", "M2", "VIAx-CO"),
    ("V1", "M2", "M1", "VIAx-CO"),
]

plane_rectangle = ("Ground", "GND", um(0.0), um(0.0), um(540), um(540))

ring = [um(110, 110), um(110, 430), um(430, 430), um(430, 110)]
ring_holes = [um(120, 120), um(120, 420), um(420, 420), um(420, 120)]
rings_split = [
    um(110, 110),
    um(110, 430),
    um(430, 430),
    um(430, 110),
    um(286.7, 110),
    um(286.7, 120),
    um(420, 120),
    um(420, 420),
    um(120, 420),
    um(120, 120),
    um(254.36, 120),
    um(254.36, 110),
]
rings = [
    ("CO", "GND_RING", ring, ring_holes),
    ("M1", "GND_RING", ring, ring_holes),
    ("V1", "GND_RING", ring, ring_holes),
    ("M2", "GND_RING", ring, ring_holes),
    ("V2", "GND_RING", ring, ring_holes),
    ("M3", "GND_RING", ring, ring_holes),
    ("V3", "GND_RING", ring, ring_holes),
    ("M4", "GND_RING", ring, ring_holes),
    ("V4", "GND_RING", ring, ring_holes),
    ("M5", "GND_RING", ring, ring_holes),
    ("V5", "GND_RING", ring, ring_holes),
    ("M6", "GND_RING", ring, ring_holes),
    ("V6", "GND_RING", ring, ring_holes),
    ("M7", "GND_RING", ring, ring_holes),
    ("V7", "GND_RING", ring, ring_holes),
    ("M8", "GND_RING", ring, ring_holes),
    ("V8", "GND_RING", rings_split, []),
    ("M9", "GND_RING", rings_split, []),
]

coils = [
    um(270.1, 110),
    um(270.1, 180.98),
    um(353.115, 180.98),
    um(353.115, 359.02),
    um(187.085, 359.02),
    um(187.085, 192.99),
    um(341.105, 192.99),
    um(341.105, 347.01),
    um(199.095, 347.01),
    um(199.095, 205),
    um(329.1, 205),
    um(329.1, 335),
    um(265.1, 335),
]

terminals = [
    (("SPIRAL", "P1", um(270.1, 110), "M9"), ("SPIRAL", "P1ref", um(270.1, 110), "M8")),
    (
        ("SPIRAL", "P2", um((265.1 + 275.1) / 2, 430), "OVERPASS"),
        ("SPIRAL", "P2ref", um((265.1 + 275.1) / 2, 430), "M9"),
    ),
]


class SpiralInductor(BaseExample):
    def run(self, path):
        with self.database(path):
            print("spiral inductor begin")
            self.create_stackup()
            self.create_ground()
            self.create_inductor()
            self.create_simsetup()
            print("spiral inductor end")

    def create_stackup(self):
        print("creating stackup")
        self.create_materials()
        self.create_layers()

    def create_ground(self):
        print("creating ground plane")
        self.create_rectangle(*plane_rectangle)

        print("creating ground ring")
        for layer_name, net_name, vertices, holes in rings:
            self.create_polygon(layer_name, net_name, vertices, holes)

    def create_inductor(self):
        print("creating inductor")
        self.create_coil()
        self.create_coil_feed()
        self.set_extents()

    def create_simsetup(self):
        print("creating simulation setup")
        self.create_adaptive_settings()

    def create_materials(self):
        print("creating materials")
        for name, properties in materials:
            self.create_material(
                name,
                {
                    MaterialProperty.CONDUCTIVITY: properties[0],
                    MaterialProperty.PERMITTIVITY: properties[1],
                    MaterialProperty.PERMEABILITY: properties[2],
                    MaterialProperty.DIELECTRIC_LOSS_TANGENT: properties[3],
                },
            )

    def create_layer_collection(self, layers):
        print("creating layer collection")
        lc = LayerCollection.create(LayerCollectionMode.OVERLAPPING)
        lc.add_layers(layers)
        # weighted_capacitance = 2
        # lc.simplify_dielectrics_for_phi(db, 5e-6, weighted_capacitance)
        self.layout.layer_collection = lc
        return lc

    def create_layers(self):
        print("creating layers")
        dls = self.create_dielectric_layers()
        sls = self.create_signal_layers()
        vls = self.create_via_layers()

        self.create_layer_collection(dls + sls + vls)

    def create_dielectric_layers(self):
        print("creating dielectric layers")
        layers = []
        elevation = 0.0
        for name, material_name, thickness in reversed(dielectrics):
            layer = StackupLayer.create(
                name, LayerType.DIELECTRIC_LAYER, um(thickness), um(elevation), material_name
            )
            layers.append(layer)
            elevation += thickness
        return layers

    def create_signal_layers(self):
        print("creating signal layers")
        layers = []
        for name, material_name, thickness, elevation in reversed(signals):
            layer = StackupLayer.create(
                name, LayerType.SIGNAL_LAYER, um(thickness), um(elevation), material_name
            )
            layers.append(layer)
            if name == "Ground":
                layer.negative = True

        return layers

    def create_via_layers(self):
        print("creating via layers")
        layers = []
        for name, upper_name, lower_name, material_name in vias:
            vl = ViaLayer.create(name, lower_name, upper_name, material_name)
            layers.append(vl)

        return layers

    def create_coil(self):
        print("creating coil")
        coil_path = self.create_path("M9", "SPIRAL", um(10), coils)
        self.create_material(
            "Coil_Mat",
            {
                MaterialProperty.CONDUCTIVITY: 3.7e7,
                MaterialProperty.PERMITTIVITY: 1,
                MaterialProperty.PERMEABILITY: 1,
                MaterialProperty.DIELECTRIC_LOSS_TANGENT: 0,
            },
        )
        coil_path.set_hfss_prop("Coil_Mat", True)

    def create_coil_feed(self):
        print("creating coil feed")
        self.create_rectangle("OVERPASS", "SPIRAL", um(265.1), um(330), um(275.1), um(430))
        polygon = PolygonData(
            [um(265.1, 330), um(265.1, 340), um(275.1, 340), um(275.1, 330)], closed=True
        )
        ViaGroup.create_with_outline(self.layout, polygon, 0.5, "VOVERPASS")
        self.create_point_terminals()

    def create_point_terminals(self):
        print("creating point terminals")

        for t, t_ref in terminals:
            terminal = self.create_point_terminal(*t)
            terminal_ref = self.create_point_terminal(*t_ref)
            terminal.reference_terminal = terminal_ref

    def set_extents(self):
        print("setting HFSS extents")
        hfss_info = HfssExtentInfo(
            dielectric_extent_type=HFSSExtentInfoType.CONFORMING,
            dielectric=(0.0, False),
            honor_user_dielectric=True,
            airbox_truncate_at_ground=True,
            airbox_horizontal=(0.15, False),
            airbox_vertical_positive=(0.5, False),
            airbox_vertical_negative=(0.15, False),
            is_pml_visible=True,
            operating_frequency=0,
        )
        self.set_hfss_extents(hfss_info)

    def create_adaptive_settings(self):
        print("creating adaptive settings")
        setup = HfssSimulationSetup.create(self.cell, "HFSS Setup 1")
        general_settings = setup.settings.general
        general_settings.single_frequency_adaptive_solution = SingleFrequencyAdaptiveSolution(
            max_delta="0.005", max_passes=20
        )
        general_settings.save_fields = True
        setup.mesh_operations = [
            SkinDepthMeshOperation(
                name="SPIRAL_M9", net_layer_info=[("SPIRAL", "M9", False)], num_layers="3"
            )
        ]
        sweep_data = SweepData(
            "Sweep 1", FrequencyData(Distribution.LIN, "0GHz", "30GHz", "0.01GHz")
        )
        sweep_data.interpolation_data.fast_sweep = True
        setup.sweep_data = [sweep_data]


def test_spiral_inductor():
    with session(settings.server_exe_dir(), 50051):
        SpiralInductor().run(r"spiral_inductor.aedb")



"""EDB-Core Python package for the Electronics Database (EDB) format."""

import os
import sys

# Ensure the package directory is on sys.path so that rpc_executor.pyd
# (placed alongside this package at install time) can be imported.
_pkg_dir = os.path.dirname(os.path.abspath(__file__))
if _pkg_dir not in sys.path:
    sys.path.append(_pkg_dir)

try:
    import importlib.metadata as importlib_metadata
except ModuleNotFoundError:
    import importlib_metadata

__version__ = importlib_metadata.version("ansys-edb-core")



"""SIWavePSI simulation setup."""
from __future__ import annotations

from typing import TYPE_CHECKING

from ansys.edb.core.simulation_setup.simulation_setup import SimulationSetup, SimulationSetupType
from ansys.edb.core.simulation_setup.siwave_psi_simulation_settings import (
    SIWavePSISimulationSettings,
)

if TYPE_CHECKING:
    from ansys.edb.core.layout.cell import Cell


class SIWavePSISimulationSetup(SimulationSetup):
    """Represents SIWavePSI simulation setup data."""

    @classmethod
    def create(cls, cell: Cell, name: str) -> SIWavePSISimulationSetup:
        """Create a SIWavePSI simulation setup.

        Parameters
        ----------
        cell : :class:`.Cell`
            Cell to create the simulation setup in.
        name : str
            Name of the simulation setup.

        Returns
        -------
        SIWavePSISimulationSetup
            SIWavePSI simulation setup created.
        """
        return super()._create(cell, name, SimulationSetupType.SI_WAVE_PSI)

    @property
    def settings(self) -> SIWavePSISimulationSettings:
        """:class:`.SIWavePSISimulationSettings`: Simulation settings of the SIWavePSI simulation setup."""
        return SIWavePSISimulationSettings(self)



"""Sphinx documentation configuration file."""
from datetime import datetime
import os
import sys

from ansys_sphinx_theme import (
    ansys_favicon,
    ansys_logo_white,
    ansys_logo_white_cropped,
    get_version_match,
    latex,
    pyansys_logo_black,
    watermark,
)
from sphinx.builders.latex import LaTeXBuilder

from ansys.edb.core import __version__

LaTeXBuilder.supported_image_types = ["image/png", "image/pdf", "image/svg+xml"]

# Project information
project = "ansys-edb-core"
copyright = f"(c) {datetime.now().year} ANSYS, Inc. All rights reserved"
author = "ANSYS, Inc."
release = version = __version__
cname = os.getenv("DOCUMENTATION_CNAME", "<DEFAULT_CNAME>")

# use the default pyansys logo
html_logo = pyansys_logo_black
html_theme = "ansys_sphinx_theme"
html_short_title = html_title = "PyEDB-Core"

html_context = {
    "github_user": "ansys",
    "github_repo": "pyedb-core",
    "github_version": "develop",
    "doc_path": "doc/source",
    "pyansys_tags": [
        "Electronics"
    ]
}

html_theme_options = {
    "switcher": {
        "json_url": f"https://{cname}/versions.json",
        "version_match": get_version_match(__version__),
    },
    "check_switcher": False,
    "github_url": "https://github.com/ansys/pyedb-core",
    "show_prev_next": False,
    "show_breadcrumbs": True,
    "collapse_navigation": True,
    "use_edit_page_button": True,
    "additional_breadcrumbs": [
        ("PyAnsys", "https://docs.pyansys.com/"),
    ],
}

html_favicon = ansys_favicon

html_static_path = ["_static"]

# additional logos for the latex coverpage
latex_additional_files = [watermark, ansys_logo_white, ansys_logo_white_cropped]

# change the preamble of latex with customized title page
# variables are the title of pdf, watermark
latex_elements = {"preamble": latex.generate_preamble(html_title)}

# Sphinx extensions
extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.autosummary",
    "numpydoc",
    "sphinx.ext.intersphinx",
    "sphinx_copybutton",
]

# Intersphinx mapping
intersphinx_mapping = {
    "python": (f"https://docs.python.org/{'.'.join((str(i) for i in sys.version_info[:2]))}", None),
    # kept here as an example
    # "scipy": ("https://docs.scipy.org/doc/scipy/reference", None),
    # "numpy": ("https://numpy.org/devdocs", None),
    # "matplotlib": ("https://matplotlib.org/stable", None),
    # "pandas": ("https://pandas.pydata.org/pandas-docs/stable", None),
    # "pyvista": ("https://docs.pyvista.org/", None),
}

# numpydoc configuration
numpydoc_show_class_members = False
numpydoc_xref_param_type = True
numpydoc_xref_ignore = {"of"}

# Consider enabling numpydoc validation. See:
# https://numpydoc.readthedocs.io/en/latest/validation.html#
numpydoc_validate = True
numpydoc_validation_checks = {
    "GL06",  # Found unknown section
    "GL07",  # Sections are in the wrong order.
    "GL08",  # The object does not have a docstring
    "GL09",  # Deprecation warning should precede extended summary
    "GL10",  # reST directives {directives} must be followed by two colons
    "SS01",  # No summary found
    # "SS02",  # Summary does not start with a capital letter
    # "SS03", # Summary does not end with a period
    "SS04",  # Summary contains heading whitespaces
    # "SS05", # Summary must start with infinitive verb, not third person
    "RT02",  # The first line of the Returns section should contain only the
    # type, unless multiple values are being returned"
}

# notfound.extension
notfound_template = "404.rst"
notfound_urls_prefix = "/../"

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

# The suffix(es) of source filenames.
source_suffix = ".rst"

# The master toctree document.
master_doc = "index"

# Ignore the following links during link checking
linkcheck_ignore = ["https://www.ansys.com/products/electronics"]

# Ignore type hints in documentation (they are only used for code completion)
autodoc_typehints = "none"


from ansys.edb.core.definition.padstack_def import PadstackDef
from ansys.edb.core.definition.padstack_def_data import PadGeometryType
from ansys.edb.core.layout.cell import Cell


def test_get_hole_parameters(circuit_cell_with_padstack_def: Cell):
    padstack_def = PadstackDef.find_by_name(circuit_cell_with_padstack_def.database, "VIA050070100")
    assert not padstack_def.is_null
    padstack_def_data = padstack_def.data
    assert not padstack_def_data.is_null
    hole_parameters = padstack_def_data.get_hole_parameters()
    assert hole_parameters[0] == PadGeometryType.PADGEOMTYPE_CIRCLE
    assert len(hole_parameters[1]) == 1
    assert round(hole_parameters[1][0].double - 50e-6, 9) == 0.0



"""Via layer."""
from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ansys.edb.core.typing import ValueLike

import ansys.api.edb.v1.via_layer_pb2 as via_layer_pb2

from ansys.edb.core.inner.messages import bool_property_message, int_property_message, value_message
from ansys.edb.core.layer.stackup_layer import StackupLayer
from ansys.edb.core.session import get_via_layer_stub
from ansys.edb.core.utility.value import Value


def _via_lyr_ref_lyr_id_msg(lyr, is_upper_ref):
    """Convert to a ``ViaLayerRefLayerIdMessage`` object."""
    return via_layer_pb2.ViaLayerRefLayerIdMessage(via_layer=lyr.msg, is_upper_ref=is_upper_ref)


class ViaLayer(StackupLayer):
    """Represents a via layer."""

    @staticmethod
    def create(name, lr_layer, ur_layer, material):
        """Create a via layer.

        Parameters
        ----------
        name : str
            Name of the via layer.
        lr_layer : str
        ur_layer : str
        material : str

        Returns
        -------
        ViaLayer
            Via layer created.
        """
        params = {
            "via_layer_name": name,
            "lower_ref_layer_name": lr_layer,
            "upper_ref_layer_name": ur_layer,
            "material_name": material,
        }
        via_layer = ViaLayer(
            get_via_layer_stub().Create(via_layer_pb2.ViaLayerCreationMessage(**params))
        )
        return via_layer

    def get_ref_layer_name(self, upper_ref):
        """Get the name of the reference layer of the via layer.

        Parameters
        ----------
        upper_ref : bool
            Whether to get the name of the upper or lower reference layer.

        Returns
        -------
        str
            Name of the reference layer.
        """
        return get_via_layer_stub().GetRefLayerName(_via_lyr_ref_lyr_id_msg(self, upper_ref)).value

    def set_ref_layer(self, ref_layer, upper_ref):
        """Set the reference layer of the via layer.

        Parameters
        ----------
        ref_layer : StackupLayer
            Layer to set as the new reference layer of the via layer.
        upper_ref : bool
            Whether to set the new reference layer as the
            upper or lower reference layer.
        """
        get_via_layer_stub().SetRefLayer(
            via_layer_pb2.SetViaLayerRefLayerMessage(
                via_layer_ref_layer_id=_via_lyr_ref_lyr_id_msg(self, upper_ref),
                new_ref_layer=ref_layer.msg,
            )
        )

    @property
    def is_tsv(self) -> bool:
        """:obj:`bool`: Flag indicating if this via layer is a TSV layer."""
        return get_via_layer_stub().GetIsTSV(self.msg).value

    @is_tsv.setter
    def is_tsv(self, is_tsv: bool):
        get_via_layer_stub().SetIsTSV(bool_property_message(self, is_tsv))

    def add_oxide_layers(self, oxide_layers: tuple[ValueLike, str] | list[tuple[ValueLike, str]]):
        """Add oxide layers to the via layer.

        Parameters
        ----------
        oxide_layers: tuple of (:term:`ValueLike`, str) or list of tuple of (:term:`ValueLike`, str)
            Tuples representing the oxide layer data. The data in the tuple is of the format \
            (``thickness``, ``material``). If a single tuple is provided, one oxide layer will \
            be provided. If a list of tuples is provided, one oxide layer will be created per tuple.
        """
        _oxide_layers = [oxide_layers] if not isinstance(oxide_layers, list) else oxide_layers
        oxide_layer_msgs = []
        for oxide_layer in _oxide_layers:
            oxide_layer_msgs.append(
                via_layer_pb2.OxideLayerMessage(
                    thickness=value_message(oxide_layer[0]), material=oxide_layer[1]
                )
            )
        get_via_layer_stub().AddOxideLayers(
            via_layer_pb2.OxideLayersPropertyMessage(target=self.msg, oxide_layers=oxide_layer_msgs)
        )

    def remove_oxide_layer(self, oxide_lyr_idx: int):
        """Remove the oxide layer at the specified index.

        Parameters
        ----------
        oxide_lyr_idx: int
            Index of the oxide layer to remove.
        """
        get_via_layer_stub().RemoveOxideLayers(int_property_message(self, oxide_lyr_idx))

    def clear_oxide_layers(self):
        """Remove all oxide layers from the via layer."""
        self.remove_oxide_layer(-1)

    @property
    def num_oxide_layers(self):
        """:obj:`int`: Number of oxide layers in the via layer.

        This property is read-only.
        """
        return get_via_layer_stub().GetNumOxideLayers(self.msg).value

    @staticmethod
    def _msg_to_oxide_layer(msg: via_layer_pb2.OxideLayerMessage) -> tuple[Value, str]:
        return Value(msg.thickness), msg.material

    @property
    def oxide_layers(self):
        """:obj:`list` of :obj:`tuple` of (:term:`ValueLike`, Obj:`str`): List of oxide layers placed on the via layer.

        This property is read-only.
        """
        response = get_via_layer_stub().GetOxideLayers(int_property_message(self, -1))
        return [
            self._msg_to_oxide_layer(oxide_layer_msg) for oxide_layer_msg in response.oxide_layers
        ]

    def get_oxide_layer(self, oxide_lyr_idx: int):
        """Get the oxide layer at the specified index.

        Parameters
        ----------
        oxide_lyr_idx: int
            Index of the oxide layer to be retrieved

        Returns
        -------
        tuple of (:term:`ValueLike`, str)
        """
        return self._msg_to_oxide_layer(
            get_via_layer_stub()
            .GetOxideLayers(int_property_message(self, oxide_lyr_idx))
            .oxide_layers[0]
        )

    def set_oxide_layer_data(
        self, oxide_lyr_idx: int, thickness: ValueLike | None = None, material: str | None = None
    ):
        """Set the thickness or material of the oxide layer at the specified index.

        Parameters
        ---------
        oxide_lyr_idx: int
            Index of the oxide layer to be retrieved
        thickness: :term:`ValueLike`, default: None
            Thickness of the oxide layer. If :obj:`None`, the thickness will not be set.
        material: str, default: None
            Material of the oxide layer.  If :obj:`None`, the material will not be set.
        """
        get_via_layer_stub().SetOxideLayerData(
            via_layer_pb2.SetOxideDataLayerMessage(
                target=int_property_message(self, oxide_lyr_idx),
                oxide_layer_data=via_layer_pb2.OxideLayerMessage(
                    thickness=None if thickness is None else value_message(thickness),
                    material=material,
                ),
            )
        )
